//Copyright 2005 Aaron Cooper
/*
This file is part of JGrail.
JGrail is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

JGrail is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with JGrail; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
*/

import javax.swing.JPanel;
import javax.swing.*;

import java.awt.event.*;
import java.io.*;
import java.awt.Color; 

/**
 * Represents the filename textfield and its associated browse button.  Handles error checking and displaying
 * of the JFileChooser when its browse button has been clicked.
 * <p>
 * 
 * @author Aaron Cooper
 * @version 1.0 4-Jun-2005
 */
public class FileNamePanel extends JPanel implements ActionListener {

	private JTextField fileNameTxtFld; 
	private JButton browseButton;
	private JFileChooser fileChooser;
	private boolean mandatory; 
	
	// used to display error messages (instead of using dialog boxes)
	private StatusBar errorStatusBar; 

	public FileNamePanel(StatusBar errorStatusBar) {
		createComponents(errorStatusBar);
		setupComponents();
		addComponents();
	}
	
	protected void createComponents(StatusBar errorStatusBar) {
		setFileNameTxtFld(new JTextField(30) );
		setBrowseButton(new JButton("Browse") );
		setFileChooser(new JFileChooser() );		
		
		setErrorStatusBar(errorStatusBar);
	}
	
	protected void setupComponents() {
		setLayout(new SpringLayout() );
		
		getBrowseButton().addActionListener(this);
		
		// when error messages occur, the textfield can be highlighted with an Error Color 
		// turn the background color to its original color once the user clicks in the textfield 
		this.getFileNameTxtFld().addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if ( getFileNameTxtFld().getBackground() == Global.ERROR_HIGHLIGHT_COLOR )
					getFileNameTxtFld().setBackground(Color.WHITE);  
			}
		}); 			
	}
	
	protected void addComponents() {
		this.add(getFileNameTxtFld() );
		this.add(getBrowseButton() ); 		
		
		SpringUtilities.makeCompactGrid(this,
										1, 2, //rows, cols
										6, 6,        //initX, initY
										10, 10);       //xPad, yPad
	}
	
	// see if textfield has any text
	public boolean isEmpty() { return getFileNameTxtFld().getText().length() == 0; }
	
	// see if textfield's text has changed from the default value
	public boolean isDefaultText() { 
		return getFileNameTxtFld().getText().trim().equalsIgnoreCase(Global.DFLT_TXT_FLD_FILE_PATH);
	}		
	
	// handle clicking of the browse button
	public void actionPerformed(ActionEvent ae) {
		String text = getFileNameTxtFld().getText().trim();
		File f;

		// if the text is the default or empty, then open up the file chooser at the user's home dir
		if ( isEmpty() || isDefaultText() ) { 
		  f = new File(Global.USER_HOME_LOCATION);
		}
		
		// otherwise open up at the given path
		else {
		  f = new File(text);
		}

		// set starting directory for the file chooser
		getFileChooser().setCurrentDirectory(f);

		// show the file chooser
		int returnVal = fileChooser.showOpenDialog(this.getParent() );

		// if user selected a file, then write the path in the textfield
		if ( returnVal == JFileChooser.APPROVE_OPTION ) {
			f = getFileChooser().getSelectedFile();
			this.getFileNameTxtFld().setText(f.getAbsolutePath() );
		}
	}


	// GETTERS and SETTERS
	protected void setErrorStatusBar(StatusBar errorStatusBar) { this.errorStatusBar = errorStatusBar; }
	protected StatusBar getErrorStatusBar() { return this.errorStatusBar; }
		
	protected void setBrowseButton(JButton browseButton) { this.browseButton = browseButton; }
	protected JButton getBrowseButton() { return this.browseButton; }

	protected void setFileChooser(JFileChooser fileChooser) { this.fileChooser = fileChooser; }
	protected JFileChooser getFileChooser() { return this.fileChooser; }
	
	protected void setFileNameTxtFld(JTextField fileNameTxtFld) { this.fileNameTxtFld = fileNameTxtFld; }
	public JTextField getFileNameTxtFld() { return this.fileNameTxtFld; }

	// only show Directories, Files or Both in the file chooser
	public void setFileSelectionMode(int mode) {
		getFileChooser().setFileSelectionMode(mode);
	}
	
	// change the browse button's text
	public void setButtonText(String text) {
		getBrowseButton().setText(text);
	}
	
	// sets the textfield's value to the given filename
	public void setFileName(String filename) {
		// if empty name given, set the text to the default value
		if ( filename == null || filename.length() == 0 )
		  getFileNameTxtFld().setText(Global.DFLT_TXT_FLD_FILE_PATH);
		  
		// otherwise assign the value		  
		else
		  getFileNameTxtFld().setText(filename);
	}
	
	// returns the value of the textfield (but doesnt validate if it's an actual file)
	public String getFileName() {
		// return empty string if needed
		if ( getFileNameTxtFld().getText() == null || this.isEmpty() || this.isDefaultText() )
			return Global.EMPTY_STRING;
		// otherwise append a slash			 
		else 
			return Global.addSlash(getFileNameTxtFld().getText().trim() );
	}

	
	// returns the textfield's text value if it represents a valid filename
	// otherwise, null is returned and an error message is presented to the user
	public String getValidFileName(int validFileType) {
		String filename = this.getFileNameTxtFld().getText().trim();
		File f;
		
		// if empty value or default, then ask user to fill in a proper value
		if ( isEmpty() || isDefaultText() ) {
			this.showErrorMsg("Please enter a file in the empty text box");

			return null;
		}

		// handle the case when the file does not exist
		f = new File(filename);
		if ( !f.exists() ) {
			this.showErrorMsg("The file " + filename + " does not exist.  Please select a valid file");

			return null;							
		}
		
		// user has typed a directory name and a file is the desired type
		if ( f.isDirectory() && validFileType == JFileChooser.FILES_ONLY ) {
			this.showErrorMsg("The entry " + filename + " is a directory.  Please enter the name of a file, and not a directory");

			return null;							
		}
		
		// user has typed a filename and a directory is the desired type
		if ( f.isFile() && validFileType == JFileChooser.DIRECTORIES_ONLY ) {
			this.showErrorMsg("The entry " + filename + " is a file.  Please enter the name of a directory, and not a file");

			return null;							
		}
		
		// otherwise it appears to be a valid filename... so return it
		return Global.addSlash(filename);
	}


	// display error message on the status bar, and highlight the textfield
	protected void showErrorMsg(String msg) {
		this.getErrorStatusBar().showErrorMsg(msg);
		
		this.getFileNameTxtFld().setBackground(Global.ERROR_HIGHLIGHT_COLOR);		
	}


	public static void main(String[] args) {
		JFrame test = new JFrame();
		test.setSize(600, 600);
		
		/*
		FileNamePanel pan = new FileNamePanel();
		
		test.getContentPane().add(pan);
		test.show();
		*/
	}
}
