//Copyright 2005 Aaron Cooper
/*
This file is part of JGrail.
JGrail is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

JGrail is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with JGrail; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
*/

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;

import java.util.*;

/**
* A status bar that normally appears at the bottom of a window.
*
* @author Aaron Cooper 
* @version 1.0 June 2004 
*/

public class StatusBar extends JLabel implements ActionListener {
  /** Text that appears before an error message */
  protected static final String ERROR_MSG_HEADER = "ATTENTION - ";
  /** Text that appears before a status message */  
  protected static final String STATUS_HEADER    = "Status: ";

  private String currStatusText;
  private BlinkTimer blinkTimer;
  private Color errorMsgColor, saveMsgColor;

	/** 
	* Creates a status bar with the given text as its message.
	*
	* @param text the message to display on the status bar
	*/ 
  public StatusBar(String text) {
    super(text);

    // if not opaque, then changes to background color aren't seen
    setOpaque(true);

    // create a border
    setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED) );

    // set the text color of the messages
    setErrorMsgColor(Global.STATUS_ERROR_MSG_COLOR);
    setSaveMsgColor(Global.STATUS_SAVE_MSG_COLOR);
  }
	
	/** Creates a status bar with a default status message */ 
  public StatusBar() { this(STATUS_HEADER); }
  
	/** 
	* Sets a fixed width for the status bar 
	* 
	* @param width the fixed width for this compoment
	*/ 
  public void setWidth(int width) {
    setMinimumSize(new Dimension(width, this.getMinimumSize().height) );
    setMaximumSize(new Dimension(width, this.getMaximumSize().height) );
    setPreferredSize(new Dimension(width, this.getPreferredSize().height) );
  }

	/** 
	* Displays an error message on the status bar, using a different color for the text 
	* 
	* @param text the error message to display 
	*/ 
  public void showErrorMsg(String text) {
    setForeground(getErrorMsgColor() );

    setText(ERROR_MSG_HEADER + text);

    blink(2);
  }

	/** 
	* Displays a message when a successful save operation has occurred.  Displays the text in a different color
	*
	* @param text the save message to display 
	*/ 
  public void showSaveMsg(String text) {
    GregorianCalendar calendar = new GregorianCalendar();
    
    setForeground(getSaveMsgColor() ); 

    setText(text + " " + calendar.getTime().toString() );
  }

	
  public void setErrorMsgColor(Color c)               { this.errorMsgColor = c; }
  public void setSaveMsgColor(Color c)                { this.saveMsgColor  = c; }
  protected void setCurrStatusText(String text)       { this.currStatusText = text; }
  protected void setBlinkTimer(BlinkTimer blinkTimer) { this.blinkTimer = blinkTimer; }

  protected Color getErrorMsgColor()          { return this.errorMsgColor; }
  protected Color getSaveMsgColor()           { return this.saveMsgColor; }
  protected String getCurrStatusText()        { return this.currStatusText; }
  protected BlinkTimer getBlinkTimer()        { return this.blinkTimer; }
  

  /**
  * Causes the status bar's current message to blink the given number of times
  *
  * @param numTimes number of times the message should blink
  *
  */
  public void blink(int numTimes) {
    // retain the current message
    setCurrStatusText(getText() );

    setBlinkTimer(new BlinkTimer(numTimes, this) );
    getBlinkTimer().start();
  }

  /**
  * Performs the blinking and keeps track of the number of blinks done thus far
  *
  * @param ae the standard ActionEvent object passed by an event.
  */
  public void actionPerformed(ActionEvent ae) {
    if ( getText() == Global.EMPTY_TEXT )
      setText(getCurrStatusText() );

    else
      setText(Global.EMPTY_TEXT);

    // add one to current tally
    getBlinkTimer().incrementNumBlinks(1);

    // if we're done with the number of blinks, stop the timer
    if ( getBlinkTimer().getCurrNumBlinks() >= getBlinkTimer().getTotalBlinks() * 2 ) {
      getBlinkTimer().stop();
    }
  }

	/** 
	* Prints the size of the componet, used for debugging 
	*/ 
  private void print() {
    printSizes(getMinimumSize().height);
    printSizes(getMaximumSize().height);
    printSizes(getPreferredSize().height);
    printSizes(getSize().height);
  }

	/** 
	* Convenient method, used by print() 
	* 
	* @param size the integer value to print to the screen 
	*/ 	
  private void printSizes(int size) {
    System.out.println("size = " + size);
  }

      /** Custom Timer... stores two extra variables for keeping track of blinks */
      private class BlinkTimer extends javax.swing.Timer {
        /** the delay between blinks */
        private static final int BLINK_DELAY = 500; 

        private int totalBlinks, currNumBlinks;

        public BlinkTimer(int totalBlinks, ActionListener l) {
          super(BLINK_DELAY, l);

          this.totalBlinks   = totalBlinks;
          this.currNumBlinks = 0;
        }

        /** increment the number of blinks so far by the given amount */
        public void incrementNumBlinks(int num) { this.currNumBlinks += num; }

        /** 
        *returns the total number of blinks this timer should produce
        *
        *@return total number of blinks
        */
        public int getTotalBlinks()             { return this.totalBlinks; }

        /**
        *returns the number of blinks that have been executed thus far
        *
        *@return number of blinks done so far
        */
        public int getCurrNumBlinks()           { return this.currNumBlinks; }
      }
			
			
  /** 
  * Main function to test out functionality for this component.
  * 
  * @param args the array of arguments, provided by Java.
  */  
  public static void main(String[] args) throws Exception {
    JFrame frmTest = new JFrame();
    frmTest.setSize(600, 600);

    StatusBar bar = new StatusBar();
    bar.setText("I am a status bar");
      //bar.setWidth(frmTest.getWidth() );

    Container content = frmTest.getContentPane();

    //content.setLayout(new BoxLayout(content, BoxLayout.PAGE_AXIS) );
    content.setLayout(new BorderLayout() );
    content.add(bar, BorderLayout.SOUTH);

    bar.print();

    frmTest.show();
    bar.blink(3);
  }
}
