//Copyright 2005 Aaron Cooper
/*
This file is part of JGrail.
JGrail is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

JGrail is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with JGrail; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
*/

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

class FuncsPanel extends AbstractPanel implements ActionListener {

  protected static final int EMPTY_CMP_SIZE     = 5;
  protected static final int EMPTY_BDR_SIZE     = 5;

  protected static final String PANEL_TITLE        = "Functions";
  protected static final String UNARY_RADIO_TITLE  = "Unary";
  protected static final String BINARY_RADIO_TITLE = "Binary";
  
  private JPanel       buttonPanel; 
	private ButtonGroup  buttonGroup;
  private JRadioButton unaryRadio, binaryRadio; 

  // used in event handling... checks the appropriate button from the "right" panel (see JGrailGui)
  private JCheckBox hideFirstChkBox, hideSecondChkBox;

  // used in the event handling (see below)
  private JRadioButton prevSelectedRadio = null;

  // list of functions to populate the drop down box
  private Function[] unaryFuncs, binaryFuncs; 

  private SteppedComboBox funcCombo;

    
  public FuncsPanel(int comboMaxWidth, int comboStaticHeight, JCheckBox hideFirstChkBox, JCheckBox hideSecondChkBox,
                    Function[] unaryFuncs, Function[] binaryFuncs) {
    super(); 

    setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS) );

    setBorder(BorderFactory.createCompoundBorder(
      BorderFactory.createTitledBorder(PANEL_TITLE),
      BorderFactory.createEmptyBorder(EMPTY_BDR_SIZE, EMPTY_BDR_SIZE, EMPTY_BDR_SIZE, EMPTY_BDR_SIZE) ) 
    );
       
    // create the widgets
    createComponents(comboMaxWidth, comboStaticHeight);

    // register event handling....
    setupComponents();

    // add components to the panel     
    addComponents();

    // hold reference to checkboxes from the "right" panel -- for event handling
    setHideFirstChkBox(hideFirstChkBox);
    setHideSecondChkBox(hideSecondChkBox);

    // lists to populate drop down boxes
    setUnaryFuncs(unaryFuncs);
    setBinaryFuncs(binaryFuncs);

    // start with Unary radio button being clicked
    getUnaryRadio().doClick();
  }

  protected void createComponents(int comboMaxWidth, int comboStaticHeight) {
    setFuncCombo(new SteppedComboBox(comboMaxWidth, comboStaticHeight) );
    createPanButtons();
  }

  protected void addComponents() {
    // add the radiobuttons on top, combo on bottom, with space between them (via cmpEmpty)
    add(getButtonPanel() );
    add(Box.createRigidArea(new Dimension(0, EMPTY_CMP_SIZE) ));
    add(getFuncCombo() );
  }

  protected void createPanButtons() {
    setUnaryRadio(new JRadioButton(UNARY_RADIO_TITLE) );
    setBinaryRadio(new JRadioButton(BINARY_RADIO_TITLE) ); 

    setButtonGroup(new ButtonGroup() );
    setButtonPanel(new JPanel() ); 

		// group the buttons -- this provides default behavior 
		// (e.g. such as only one button clicked at a time)
		getButtonGroup().add(getUnaryRadio() ); 
		getButtonGroup().add(getBinaryRadio() );

    // place buttons on the left and right sides of the panel 
		getButtonPanel().setLayout(new BorderLayout() ); 

    // add buttons to the panel 
		getButtonPanel().add(getUnaryRadio(),  BorderLayout.LINE_START); 
		getButtonPanel().add(getBinaryRadio(), BorderLayout.LINE_END); 
  }

  // register event handling routine
  protected void setupComponents() {
    getUnaryRadio().addActionListener(this);
    getBinaryRadio().addActionListener(this);
  }

  // required abstract method....  
  protected void createComponents() {}

  public void changeListData(Function[] unaryFuncs, Function[] binaryFuncs) {
    DefaultComboBoxModel model;

    setUnaryFuncs(unaryFuncs);
    setBinaryFuncs(binaryFuncs);

    if ( getUnaryRadio().isSelected() )
      model = new DefaultComboBoxModel(getUnaryFuncs() );
    else
      model = new DefaultComboBoxModel(getBinaryFuncs() );
    
    getFuncCombo().setModel(model);
    getFuncCombo().repaint();    
  }

  public void actionPerformed(ActionEvent ae) {
    JRadioButton source = (JRadioButton) ae.getSource();

    // only change the drop down list data if this radio button wasn't previously selected 
    // (otherwise we're just populating with the data that is already in the list)    
    if ( source != getPrevSelectedRadio() ) {
      DefaultComboBoxModel model;

      //  get unary functions list and hide the second input panel
      if ( source == getUnaryRadio() ) {
        model = new DefaultComboBoxModel(getUnaryFuncs() );

        // only show the "first input" panel if Unary is clicked
        if ( getHideFirstChkBox().isSelected() )
          getHideFirstChkBox().doClick();

        if ( !getHideSecondChkBox().isSelected() )
          getHideSecondChkBox().doClick();
      }

      // get binary functions lists and show both input panels
      else {
        model = new DefaultComboBoxModel(getBinaryFuncs() );

        // show both input panels if Binary is clicked
        if ( getHideFirstChkBox().isSelected() )
          getHideFirstChkBox().doClick();
          
        if ( getHideSecondChkBox().isSelected() )
          getHideSecondChkBox().doClick();
      }

      // set the new list of data
      getFuncCombo().setModel(model);

      // remember which button was previously selected...
      setPrevSelectedRadio(source);
    }
  }

  
  

  protected void setUnaryFuncs(Function[] unaryFuncs) { this.unaryFuncs = unaryFuncs; }
  protected Function[] getUnaryFuncs() { return this.unaryFuncs; }

  protected void setBinaryFuncs(Function[] binaryFuncs) { this.binaryFuncs = binaryFuncs; }
  protected Function[] getBinaryFuncs() { return this.binaryFuncs; }

  protected void setFuncCombo(SteppedComboBox funcCombo) { this.funcCombo = funcCombo; }
  public SteppedComboBox getFuncCombo() { return this.funcCombo; } // PUBLIC //
  

  protected void setButtonPanel(JPanel buttonPanel) { this.buttonPanel = buttonPanel; }
  protected JPanel getButtonPanel() { return this.buttonPanel; }

  protected void setButtonGroup(ButtonGroup buttonGroup) { this.buttonGroup = buttonGroup; }
  protected ButtonGroup getButtonGroup() { return this.buttonGroup; }
  

  protected void setUnaryRadio(JRadioButton unaryRadio) { this.unaryRadio = unaryRadio; }
  protected JRadioButton getUnaryRadio() { return this.unaryRadio; }

  protected void setBinaryRadio(JRadioButton binaryRadio) { this.binaryRadio = binaryRadio; }
  public JRadioButton getBinaryRadio() { return this.binaryRadio; } // PUBLIC //

  protected void setPrevSelectedRadio(JRadioButton prevSelectedRadio) { this.prevSelectedRadio = prevSelectedRadio; }
  protected JRadioButton getPrevSelectedRadio() { return this.prevSelectedRadio; }
  

  protected void setHideFirstChkBox(JCheckBox hideFirstChkBox) { this.hideFirstChkBox = hideFirstChkBox; }
  protected JCheckBox getHideFirstChkBox() { return this.hideFirstChkBox; }

  protected void setHideSecondChkBox(JCheckBox hideSecondChkBox) { this.hideSecondChkBox = hideSecondChkBox; }
  protected JCheckBox getHideSecondChkBox() { return this.hideSecondChkBox; }
}
