//Copyright 2005 Aaron Cooper
/*
This file is part of JGrail.
JGrail is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

JGrail is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with JGrail; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
*/

import java.awt.*; 
import java.awt.event.*; 
import javax.swing.*;

/**
* The panel that contains the "unary functions-only" combo box.
* It is located in the left-side panel of the main GUI.
*
* @author Aaron Cooper June 2004
* @see ListFuncsPanel
* @see SteppedComboBox
*/

public class UnaryFuncsPanel extends AbstractPanel implements ActionListener {
  /** Empty space size between the combo box and the add button */
  protected static final int EMPTY_CMP_SIZE    = 5;
  /** Empty internal border size for the panel */
  protected static final int EMPTY_BDR_SIZE    = 5;
  /** Title for the panel */
  protected static final String PANEL_TITLE    = "Unary Functions";
  /** Title for the add button */
  protected static final String BTN_TITLE      = "Add to List";

  private JButton         addButton;
  private SteppedComboBox funcCombo;
  private JList           funcList;

  /**
    * Constructs the panel with its combo box's data populated via the array of Functions
    * <p>
    * SteppedComboBox.java uses comboMaxWidth and comboStaticHeight in its constructor.
    *
    * @param comboMaxWidth   - max width the combo box can grow
    * @param comboStaticHeight - the exact height for the combo box
    * @param funcList - list of functions that will be executed, used in event-handling to add new functions
    * @param unaryFuncs - list of data to populate the combo box
    *
    */
  public UnaryFuncsPanel(int comboMaxWidth, int comboStaticHeight, JList funcList, Function[] unaryFuncs) {
    super(); 

      // set background color -- debugging
      //setBackground(Color.MAGENTA); 

    setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS) ); 

    setBorder(BorderFactory.createCompoundBorder(
      BorderFactory.createTitledBorder(PANEL_TITLE),
      BorderFactory.createEmptyBorder(EMPTY_BDR_SIZE, EMPTY_BDR_SIZE, EMPTY_BDR_SIZE, EMPTY_BDR_SIZE) )
    );

    createComponents(comboMaxWidth, comboStaticHeight, unaryFuncs);
    setupComponents();
    addComponents();

    // source list that will have values added via the Add Button
    setFuncList(funcList);
  }

  
  /**
    * Returns the combo box that contains names of unary functions
    *<p>
    * It is public because it is needed by external classes for GUI event-handling
    *
    *@return the combo box containing names of unary functions
    */
  public SteppedComboBox getFuncCombo() { return this.funcCombo; }


  /**
    * Returns the button to add data from the combo box to the list of functions to execute
    *
    *@return the add button 
    */
  protected JButton      getAddButton()    { return this.addButton; }

  
  /**
    * Returns the list that contains functions that will be executed
    *
    *@return the list of functions to execute  
    */
  protected JList        getFuncList()     { return this.funcList; }

  
  /**
    * Assigns the panel's combo box of available unary functions.
    *
    *@param aComboBox the combo box to use for this panel
    */  
  protected void setFuncComboBox(SteppedComboBox funcCombo) { this.funcCombo = funcCombo; }

  
  /**
    * Assigns the panel's button to add functions to the list of functions.
    *
    *@param aButton the button to use for this panel 
    */
  protected void setAddButton(JButton addButton)              { this.addButton    = addButton; }

  
  /**
    * Assigns the list that will be appended with functions from the combo box
    *
    *@param aList the list to use for this panel 
    */
  protected void setFuncList(JList funcList)                   { this.funcList     = funcList; }

  
  /**
    * Change the combo box's data
    *
    *@param unaryFuncs the data to populate the combo box
    */  
  public void changeListData(Function[] unaryFuncs) {
    DefaultComboBoxModel model;
    model = new DefaultComboBoxModel(unaryFuncs);

    getFuncCombo().setModel(model);
    getFuncCombo().repaint(); 
  }

  
  /**
    * Creates the GUI components for the panel
    *
    *@param comboMaxWidth max width the combo box can grow
    *@param comboStaticHeight the exact height for the combo box
    *@param unaryFuncs list of data to populate the combo box
    */
  protected void createComponents(int comboMaxWidth, int comboStaticHeight, Function[] unaryFuncs) {
    setFuncComboBox(new SteppedComboBox(unaryFuncs, comboMaxWidth, comboStaticHeight) );
    setAddButton(new JButton(BTN_TITLE) );
  }

  
  /** Modifies some properties of the GUI components  */
  protected void setupComponents() {
    getAddButton().setAlignmentX(CENTER_ALIGNMENT);
    getAddButton().addActionListener(this);
  }


  /** Adds the GUI components to the panel */
  protected void addComponents() {
    add(getFuncCombo() );
    add(Box.createRigidArea(new Dimension(0, EMPTY_CMP_SIZE) ));
    add(getAddButton() );
  }


  /** Required abstract method, contains no implementation */
  protected void createComponents() {}


  /**
    * Event that adds the currently selected combo box item to the list of functions
    *
    *@param ae ActionEvent that responds to the Add button clicks
    */
  public void actionPerformed(ActionEvent ae) {
    if ( getFuncCombo().getSelectedIndex() != Global.NOT_FOUND ) {
      DefaultListModel model = (DefaultListModel) getFuncList().getModel();
      Function item = (Function) getFuncCombo().getSelectedItem();

      model.addElement(item);
    }
  }
}
