import java.awt.*;
import java.awt.event.*; 
 
import javax.swing.*; 
import javax.swing.table.*;
import javax.swing.border.BevelBorder;

import java.util.Vector;
import java.util.Enumeration;
import java.io.File;

/**
* The dialog window that allows the user to change program settings, such as 
* the values displayed in the Unary/Binary combo boxes and global variables (e.g. path to save files).
* <p>
* To display this dialog window, the user clicks on the Configure option from the JGrail menu bar.
*
* @author Aaron Cooper
* @version 1.0 May-June 2004
*/

public class ConfDialog extends JDialog implements ActionListener {
  /** Title for the Dialog Window */ 
	protected static final String TITLE   = "Configuration -- Modify Available Functions"; 
  /** Default size for the Dialog Window */ 		
  protected static final int DFLT_SIZE  = 600;

  /** Constant to represent that the Save button was clicked by the user */ 
  public static final int SAVE_BUTTON   = 0;
  /** Constant to represent that the Cancel button was clicked by the user */ 
  public static final int CANCEL_BUTTON = 1;
  
  // tab indices
  protected static final int UNARY_TAB_INDEX = 0; 
  protected static final int BINARY_TAB_INDEX = 1; 
  protected static final int PATHS_TAB_INDEX = 2;
  protected static final int TOOLS_TAB_INDEX = 3;
  


  //// INSTANCE VARIABLES
  private JTabbedPane       confPane; 
  private ConfTablePanel    unaryPanel, binaryPanel; 
  private ConfPathPanel     pathPanel; 
  private ConfToolsPanel 	toolsPanel; 
  
  private int               clickedButton; 

  private JPanel    buttonPanel, mainPanel; 
  private JButton   saveButton, cancelButton; 
  private StatusBar statusBar;
  private String    invFilePathErrorMsg;

  private ConfFile  confFile;

  
  /**
  * Constructs the Dialog Window with a JFrame as its parent, and program settings are obtained from the configuration file. 
  * 
  * @param parent JFrame which is the parent to this window
  * @param confFile the configuration file which holds program settings
  */
	public ConfDialog(JFrame parent, ConfFile confFile) {
		super(parent, TITLE, true); 

    setSize(parent);
    setConfFile(confFile); 

    // create the components and add them
    createComponents();
    setupComponents();
    addComponents();
	}

  
  // returns the list of functions in the unary (or binary) functions tab 
  // from the Configuration Dialog
  private Vector getFunctions(ConfTablePanel sourcePanel) {
	//	unselect cells if they are currently being edited
  	if ( sourcePanel.isEditing() )
  		sourcePanel.setEditing(false);
  		
	// now return a vector of the table's data  		
	return sourcePanel.getTableData();  		
  }
  
  
  	/**
  * Handles the clicking events of the Save and Cancel buttons.
  * <p>
  * Save button saves the changes made by the user to the configuration file.  Cancel button returns the user to JGrail screen.
  * 
  * @param ae the ActionEvent object provided by Java
  */
  public void actionPerformed(ActionEvent ae) {
    JButton source = (JButton) ae.getSource();

    // handle Save
    if ( source == getSaveButton() ) {
		// validate the data on the panels before trying to save
		if ( validation() )
			handleSave();  
    }
    
    // otherwise cancel button has been clicked
    else {
      handleClose();
    }
  }
  
  private boolean validation() {
  	boolean success = false; // assume unsuccessful 
  	
  	// if unary func panel doesn't validate correctly, show error msg 
  	if ( !unaryPanel.validation() ) {
  		 this.getStatusBar().showErrorMsg(ErrorMessages.TBL_INVALID_FUNC);
  		 
  		 // make sure the unary panel tab is visible
		 getConfPane().setSelectedIndex(UNARY_TAB_INDEX); 
  	}
  	
	// now validate the binary funcs 
	else if ( !binaryPanel.validation() ) {
		 this.getStatusBar().showErrorMsg(ErrorMessages.TBL_INVALID_FUNC);
  		 
		 // make sure the binary panel tab is visible
		 getConfPane().setSelectedIndex(BINARY_TAB_INDEX); 
	}

	// validate the paths	
	else if ( !pathPanel.validation() ) 
	  getConfPane().setSelectedIndex(PATHS_TAB_INDEX);     
	
	// and finally, the tools 
	else if ( !toolsPanel.validation() ) {
	  this.getStatusBar().showErrorMsg(ErrorMessages.TBL_INVALID_TOOL);
	  
	  getConfPane().setSelectedIndex(TOOLS_TAB_INDEX);
	}	  

  	// otherwise successful validation
  	else 
  		success = true;
  		
  	return success; 
  }


  // ensure that the save path can be written to
  protected boolean isWritablePath(String savePath) {
	// ensure save location is writeable
	if ( !isWriteable(savePath) ) {
		// show error message	
		getStatusBar().showErrorMsg(ErrorMessages.DIR_NOT_WRITABLE); 

		// change the background to grab user's attention		
		getPathPanel().getSaveFileNamePanel().getFileNameTxtFld().setBackground(Global.ERROR_HIGHLIGHT_COLOR);
  
		// ensure path tab is visible
		getConfPane().setSelectedIndex(PATHS_TAB_INDEX);

		// don't proceed any further	  		
		return false;
	}
	
	return true; 
  }


  
  // deals with the Save event 
  private void handleSave() {
	boolean saveSuccess;
	Vector vUnary, vBinary;
	String execPath, savePath; 	
	Function[] tools;	  

	// remember that the save button was clicked (used by showDlg() )
	setClickedButton(SAVE_BUTTON);

	// get the list of unary and binary functions
	vUnary   = getFunctions(getUnaryPanel() ); 
	vBinary  = getFunctions(getBinaryPanel() ); 

	// get the paths 			
	execPath = getPathPanel().getExecPath();
	savePath = getPathPanel().getSavePath();

	// get the list of external tools from the Tools tab
	tools = this.getToolsPanel().getTools();
		  
	// re-assign the tools' execpath to the value entered by the user 
	for (int i=0; i < tools.length; i++) {
		if ( tools[i] != null)
	  		tools[i].setExecPath(execPath);
	}
	
	// if the savepath is not empty or default text, make sure it's writeable
	// if not, then don't proceed further	
	if ( !getPathPanel().getSaveFileNamePanel().isEmpty() && !getPathPanel().getSaveFileNamePanel().isDefaultText() ) {
		if ( !isWritablePath(savePath) )
			return; 
	}
		
	// try to save the changes
	saveSuccess = getConfFile().saveChanges(vUnary, vBinary, execPath, savePath, tools);

	// display status bar info
	if ( saveSuccess ) {
	  getStatusBar().showSaveMsg(Global.SAVE_FILE_MSG + ConfFile.CONF_FULL_PATH); 

	  // no new changes, because we just saved...so reset
	  getPathPanel().setModified(false);
	  getUnaryPanel().setModified(false);
	  getBinaryPanel().setModified(false);
	}
	else
	  getStatusBar().showErrorMsg(getConfFile().getSaveErrorMsg() );
  }
	

  /**
  * If changes have been made to the Dialog Window's values, the system prompts the user for confirmation to close without 
  * saving their changes.  Otherwise, the Dialog Window closes and returns the user to the JGrail screen.
  */
  private void handleClose() {
    setClickedButton(CANCEL_BUTTON); 

      System.out.println("handle close");

    if ( getUnaryPanel().isModified() || getBinaryPanel().isModified() || getPathPanel().isModified() ) {
      int ans = JOptionPane.showConfirmDialog(this, "Close without saving your changes?", "Confirm", JOptionPane.YES_NO_OPTION);

      if ( ans == JOptionPane.NO_OPTION ) {
          System.out.println("handleClose()... no_option selected");
        return;
      }
    }
    this.dispose();
  }

	
  /**
  * Displays the dialog and returns which button was clicked to close it (save or cancel)
  * 
  * @return an integer represented which button was clicked to close the Dialog Window.  Value is either SAVE_BUTTON or CANCEL_BUTTON
  */
  public int showDlg() {
    super.show();

    return getClickedButton();
  }
  

  /** 
  * Determines if a file is writeable. 
  * 
  * @param filename 
  * @return boolean value representing whether the file is writeable.
  */ 
  private boolean isWriteable(String filename) {
    File f = new File(filename);

    try {
      // check that it exists and it's writeable
      if ( f.exists() && f.canWrite() )
        return true;

      // assume unsuccessful
      return false;
    }
    catch (SecurityException se) {
      return false;
    }
  }

	
 /**
  * Determines if a directory exists. 
  * 
  * @param dirPath the directory location to test for existence
  * @return boolean value representing the directory's existence
  */ 
  private boolean dirExists(String dirPath) {
    File f = new File(dirPath);

    try {
      // file doesn't exist
      if ( !f.exists() ) {
        setInvFilePathErrorMsg(dirPath + " does not exist");
      }
      // file isn't a directory
      else if ( !f.isDirectory() ) {
        setInvFilePathErrorMsg(dirPath + " is not a directory");
      }
      // otherwise good to go...
      else {
        return true;
      }

      // assume unsuccessful
      return false;
    }
    catch (SecurityException se) {
      setInvFilePathErrorMsg(dirPath + " does not exist, due to a security exception ");
      return false;
    }
  }


 /**
  * Creates the GUI widgets for this Dialog Window
  */ 
  protected void createComponents() {
    setConfPane(new JTabbedPane() );

    setUnaryPanel(new ConfTablePanel(this.getWidth() - 450, this.getHeight() - 300, getConfFile().getUnaryFuncs() ));
    setBinaryPanel(new ConfTablePanel(this.getWidth() - 100, this.getHeight() - 250, getConfFile().getBinaryFuncs() ));

    setButtonPanel(new JPanel() );
    setMainPanel(new JPanel() );
    
    setSaveButton(new JButton("Save") );
    setCancelButton(new JButton("Cancel") );

    // status bar
    setStatusBar(new StatusBar() );
    
    // pathpanel needs access to the statusbar... so create it last
	setPathPanel(new ConfPathPanel(this) );
	
	setToolsPanel(new ConfToolsPanel(getConfFile().getToolsArray() ));
  }

	
  /** 
  * Assigns values to properties of the GUI widgets contained in this Dialog Window
  */  	
  protected void setupComponents() {
    getMainPanel().setLayout(new GridBagLayout() );
      //panMain.setBorder(BorderFactory.createLineBorder(Color.RED) );
      //panMain.setBackground(Color.PINK);
    
    getConfPane().addTab("Unary Functions", getUnaryPanel() );
    getConfPane().addTab("Binary Functions", getBinaryPanel() );
    getConfPane().addTab("Default Paths", getPathPanel() );
    getConfPane().addTab("External Tools", getToolsPanel() ); 

      //panButtons.setBorder(BorderFactory.createLineBorder(Color.BLUE) );
      //panBottom.setLayout(new BorderLayout() );

    // make the Save and Cancel buttons the same size
    getSaveButton().setMinimumSize(getCancelButton().getMinimumSize() );
    getSaveButton().setPreferredSize(getCancelButton().getPreferredSize() );
    getSaveButton().setMaximumSize(getCancelButton().getMaximumSize() );          

    // add buttons to panel, with empty space between them
    getButtonPanel().setLayout(new BoxLayout(getButtonPanel(), BoxLayout.LINE_AXIS) );
    getButtonPanel().add(getSaveButton() );
    getButtonPanel().add(Box.createRigidArea(new Dimension(50, 0) ));
    getButtonPanel().add(getCancelButton() );

    // register event handling
    getSaveButton().addActionListener(this);
    getCancelButton().addActionListener(this);

    // register a new event for handling closing via the "X" button
    // just calls handleClose() above.... (same as clicking cancel button)
    this.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent we) {
        handleClose();
      }
    });
    // disable auto-closing 
    setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
	}

	
  /** 
  * Assigns values to properties of a GridBagLayout manager. 
  * <p>
  * More information on how to use GridBagLayout can be obtained at 
  * http://java.sun.com/docs/books/tutorial/uiswing/layout/gridbag.html
  */ 
  protected void setupGridLayout() {
    GridBagConstraints c = new GridBagConstraints();

    // TABBED PANE //
    c.gridx = 0;
    c.gridy = 0;

    c.weightx = 1.0;
    c.weighty = 1.0;

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.fill      = GridBagConstraints.BOTH;
    getMainPanel().add(getConfPane(), c);
    
    // BUTTONS //
    c.gridx = 0;
    c.gridy = 1;

    c.weightx = 0.0;    
    c.weighty = 0.0;

    c.insets  = new Insets(30, 0, 10, 0);
        
    c.gridwidth = GridBagConstraints.REMAINDER;    
    c.fill      = GridBagConstraints.NONE;
    getMainPanel().add(getButtonPanel(), c);

    // STATUS BAR //
    c.gridx = 0;
    c.gridy = 2;

    c.weightx = 1.0;    
    c.weighty = 0.0;

    c.insets  = new Insets(0, 0, 0, 0);
    
    c.gridwidth = GridBagConstraints.REMAINDER;        
    c.fill      = GridBagConstraints.HORIZONTAL;
    c.anchor    = GridBagConstraints.PAGE_END;
    getMainPanel().add(getStatusBar(), c);
  }

	
  
  /** 
  * Adds the GUI widgets to this Dialog Window 
  */  	
  protected void addComponents() {
    setupGridLayout();
    setContentPane(getMainPanel() );
  }


  /** 
  * Sets the size of this Dialog Window, in proportion to its parent.
  * 
  * @param parent Parent frame for this Dialog Window.  If parent is null, creates a Dialog with default size values.
  */
  private void setSize(JFrame parent) {
    // try setting size relative to parent frame
    if ( parent != null )
      setSize(parent.getWidth() - 20, parent.getHeight() - 20);
    else
      setSize(DFLT_SIZE, DFLT_SIZE);
  }

	
  protected void setButtonPanel(JPanel buttonPanel) { this.buttonPanel = buttonPanel; }
  protected JPanel getButtonPanel() { return this.buttonPanel; }

  protected void setMainPanel(JPanel mainPanel) { this.mainPanel = mainPanel; }
  protected JPanel getMainPanel() { return this.mainPanel; }

  protected void setSaveButton(JButton saveButton) { this.saveButton = saveButton; }
  protected JButton getSaveButton() { return this.saveButton; }

  protected void setCancelButton(JButton cancelButton) { this.cancelButton = cancelButton; }
  protected JButton getCancelButton() { return this.cancelButton; }

  protected void setStatusBar(StatusBar statusBar) { this.statusBar = statusBar; }
  protected StatusBar getStatusBar() { return this.statusBar; }

  protected void setConfFile(ConfFile confFile) { this.confFile = confFile; }
  protected ConfFile getConfFile() { return this.confFile; }

  protected void setInvFilePathErrorMsg(String msg) { this.invFilePathErrorMsg = msg; }
  protected String getInvFilePathErrorMsg() { return this.invFilePathErrorMsg; }


  protected void setClickedButton(int clickedButton) { this.clickedButton = clickedButton; }
  protected int getClickedButton() { return this.clickedButton; }


  protected void setConfPane(JTabbedPane confPane) { this.confPane = confPane; }
  protected JTabbedPane getConfPane() { return this.confPane; }

  protected void setUnaryPanel(ConfTablePanel unaryPanel) { this.unaryPanel = unaryPanel; }
  protected ConfTablePanel getUnaryPanel() { return this.unaryPanel; }

  protected void setBinaryPanel(ConfTablePanel binaryPanel) { this.binaryPanel = binaryPanel; }
  protected ConfTablePanel getBinaryPanel() { return this.binaryPanel; }

  protected void setPathPanel(ConfPathPanel pathPanel) { this.pathPanel = pathPanel; }
  protected ConfPathPanel getPathPanel() { return this.pathPanel; }

  protected void setToolsPanel(ConfToolsPanel confToolsPanel) { this.toolsPanel = confToolsPanel; }
  protected ConfToolsPanel getToolsPanel() { return this.toolsPanel; }



  /** 
  * Main function to test out functionality for this component.
  * 
  * @param args the array of arguments, provided by Java.
  */  
	public static void main(String[] args) throws Exception {
    JFrame frmTest = new JFrame();
    frmTest.setSize(800, 600);

    ConfFile confFile = new ConfFile();
    confFile.parseConfFile();

    ConfDialog dlgConf = new ConfDialog(frmTest, confFile);

    int ans = dlgConf.showDlg();

    System.out.println("ans = " + ans);
	}
}
