//Copyright 2005 Aaron Cooper
/*
This file is part of JGrail.
JGrail is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

JGrail is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with JGrail; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
*/

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

import java.io.*;

/** 
* Displays textfields to modify the program settings for the EXEC_PATH and SAVE_PATH variables.  This panel 
* is displayed as one of the tabs in ConfDialog.java
*
* @author Aaron Cooper
* @version 1.0 May-June 2004
*/ 

public class ConfPathPanel extends AbstractPanel implements DocumentListener {
  /** Label for the EXEC_PATH textfield */ 
  protected static final String EXEC_LABEL            = "Function Folder Location: ";
  /** Label for the SAVE_PATH textfield */ 
  protected static final String SAVE_LABEL            = "Save Folder Location: ";

  /** Label for the EXEC_PATH Browse Button */ 	
  protected static final String EXEC_BROWSE_BTN_TITLE = "Browse"; 
  /** Label for the SAVE_PATH Browse Button */ 	
  protected static final String SAVE_BROWSE_BTN_TITLE = "Browse"; 
  
  /** Default title bar value for the JFileChooser Dialog Window */ 
  protected static final String FILE_CHOOSER_TITLE   = "Please Select A Folder";
  /** Default Textfield size */ 
  protected static final int TEXTFIELD_SIZE = 50;

  //// INSTANCE VARIABLES
  private JLabel       execLabel, saveLabel;
  private FileNamePanel saveFileNamePanel, execFileNamePanel;

  private boolean isModified;
  
  
  /** 
  * Creates the panel with the textfield values set to the given arguments.
  *
  * @param execPath the location where the Grail+ executables are located 
  * @param savePath the location where the user stores his output
  */ 
	public ConfPathPanel(ConfDialog parent) {
    setLayout(new SpringLayout() );    
    setModified(false);     

    createComponents(parent);
    setupComponents();
    addComponents();
    
	// set the textfields to the paths
	getExecFileNamePanel().setFileName(parent.getConfFile().getExecPath() );
	getSaveFileNamePanel().setFileName(parent.getConfFile().getSavePath() );
	
  }
    
    // make sure valid paths are given
	public boolean validation() {
		String execPath, savePath; 
		
		// since savepath is not mandatory, if it is empty (or the default text), it's valid
		if ( getSaveFileNamePanel().isEmpty() || getSaveFileNamePanel().isDefaultText() )
			savePath = ""; 
			
		// otherwise call the getValid function			
		else 
			savePath = getSaveFileNamePanel().getValidFileName(JFileChooser.DIRECTORIES_ONLY);
			
		// if savepath was found to be null, then it is not a valid filename and we'll return unsuccess			
		if ( savePath == null ) {
				System.out.println("savePath is null....");
			return false;
		}			 		
		
		// do the same for the exec path
		execPath = getExecFileNamePanel().getValidFileName(JFileChooser.DIRECTORIES_ONLY);
			
		// unsuccessful validation			
		if ( execPath == null )
			return false; 			
			
		// otherwise valid paths have been given			
		return true;
	}
	
	
  /** Required by the DocumentListener interface. */ 
  public void changedUpdate(DocumentEvent e) {}
 
  /** 
  * Required by the DocumentListener interface.  Marks that the textfield values have changed.
  * Useful when determining whether or not the textfield values require been saved. 
  */ 
  public void insertUpdate(DocumentEvent e) { setModified(true); }
  /** 
  * Required by the DocumentListener interface.  Marks that the textfield values have changed.
  * Useful when determining whether or not the textfield values require been saved. 
  */ 
  public void removeUpdate(DocumentEvent e) { setModified(true); }


  // required abstract method
  protected void createComponents() {}
	
  /**
  * Creates the GUI widgets for this Panel
  */ 
  protected void createComponents(ConfDialog parent) {
    setExecLabel(new JLabel(EXEC_LABEL, JLabel.TRAILING) );
    setSaveLabel(new JLabel(SAVE_LABEL, JLabel.TRAILING) );
    
    setSaveFileNamePanel(new FileNamePanel(parent.getStatusBar() ));
	setExecFileNamePanel(new FileNamePanel(parent.getStatusBar() ));
  }

	
  /** 
  * Creates and sets up the SpringLayout manager. 
  * <p>
  * More information about SpringLayout can be obtained at 
  * http://java.sun.com/docs/books/tutorial/uiswing/layout/spring.html
  * <p>
  * Also, refer to SpringUtilities.java
  */ 
  protected void setupSpringLayout() {
    // add components      
    add(getExecLabel() );
	add(getExecFileNamePanel() );

    add(getSaveLabel() );
    add(getSaveFileNamePanel() );

    //Lay out the panel.
    SpringUtilities.makeCompactGrid(this,
                                    2, 2, //rows, cols
                                    6, 6,        //initX, initY
                                    10, 10);       //xPad, yPad
  }

  
	/** 
  * Assigns values to properties of the GUI widgets contained in this Panel
  */  	
  protected void setupComponents() {
    // setup the file chooser options
    getSaveFileNamePanel().setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
	getExecFileNamePanel().setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
  }

  
	/** 
  * Adds the GUI widgets to this Panel
  */  	
  protected void addComponents() {
    setupSpringLayout();
  }


  /** 
  * Determines if a string is empty
  * 
  * @param text the text to check for emptiness
  * @return boolean value, representing whether or not the string was empty
  */ 
	private boolean isEmpty(String text) {
    if ( text == null || text.trim().length() == 0 )
      return true;
    return false;
  }

  // GETTERS & SETTERS //
  public boolean isModified() { return this.isModified; }
  public void setModified(boolean isModified) { this.isModified = isModified; }
  
  public String getExecPath()  { 
  	return this.getExecFileNamePanel().getFileName(); 
  }
  
  public String getSavePath() {
	return this.getSaveFileNamePanel().getFileName(); 
  }
  
  protected void setExecLabel(JLabel execLabel) { this.execLabel = execLabel; }
  protected JLabel getExecLabel()          { return this.execLabel; }

  protected void setSaveLabel(JLabel saveLabel) { this.saveLabel = saveLabel; }
  protected JLabel getSaveLabel()          { return this.saveLabel; }

  protected void setSaveFileNamePanel(FileNamePanel saveFileNamePanel) { this.saveFileNamePanel = saveFileNamePanel; }
  protected FileNamePanel getSaveFileNamePanel() { return this.saveFileNamePanel; }

  protected void setExecFileNamePanel(FileNamePanel execFileNamePanel) { this.execFileNamePanel = execFileNamePanel; }
  protected FileNamePanel getExecFileNamePanel() { return this.execFileNamePanel; }

  
 /** 
  * Main function to test out functionality for this component.
  * 
  * @param args the array of arguments, provided by Java.
  */  
  public static void main(String[] args) throws Exception {
    JFrame frame = new JFrame();
    frame.setSize(600, 600);
      //frame.getContentPane().setLayout(new BoxLayout(frame.getContentPane(), BoxLayout.PAGE_AXIS) );

	/*
	ConfPathPanel pan = new ConfPathPanel(null, null);

    frame.setContentPane(pan);
    frame.show();
    */
  }
}









