//Copyright 2005 Aaron Cooper
/*
This file is part of JGrail.
JGrail is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

JGrail is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with JGrail; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
*/

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import java.io.*;


/**
* The graphical user interface for the dialog windows that allow the user to view or edit a finite state machine 
* using an external program (such as Xfig).  This is just the UI... see subclasses ViewDiagDialog and EditDiagDialog.
* <p>
* 
*
* @author Aaron Cooper
* @version 1.0 May 22, 2005
*/

public class DiagDialog extends JDialog implements ActionListener {
	/** Default size for the Dialog Window */ 		
	protected static final int DFLT_SIZE  = 500;
	
	/** Pixel-width of an empty component that is placed between the GUI widgets */ 
	protected static final int EMPTY_CMP_SIZE = 5;
	
	/** Size of the filename text field */ 
	protected static final int TXT_FLD_SIZE = 15; 
	
	// debugging variable
	private boolean D = false;


	// INSTANCE VARS
	private JPanel mainPanel, buttonPanel; 
	private JRadioButton firstInputRadio, secondInputRadio, outputRadio, fileNameRadio;
	private ButtonGroup buttonGroup;
	private JButton viewEditButton;

	private FileNamePanel fileNamePanel; 
	private StatusBar statusBar;
	
	// rightpanel from JGrailGui.java... needed for handling actionPerformed events
	private RightPanel rightPanel; 
	
	// determined by actionPerformed method below
	private String inputFile; 

	public DiagDialog(JFrame parent, RightPanel rightPanel, String buttonTitle) {
		super(parent, true);
		
		this.setSize(DFLT_SIZE, DFLT_SIZE);
		this.setRightPanel(rightPanel);
				
		// create the components and add them
		createComponents();
		setupComponents();
		addComponents();
		
		// set the title of the button (either View or Edit)
		getViewEditButton().setText(buttonTitle);
	}
	
	protected void createComponents() {
		// create the buttons
		setViewEditButton(new JButton() );
		
		// create the radio buttons
		setFirstInputRadio(new JRadioButton("Input1", true) );
		setSecondInputRadio(new JRadioButton("Input2") );
		setOutputRadio(new JRadioButton("Output") );
		setFileNameRadio(new JRadioButton("File") );
		
		// create the group that will house the radio buttons
		setButtonGroup(new ButtonGroup() );
		
		// create the status bar
		setStatusBar(new StatusBar() );
		
		// create the panels
		setMainPanel(new JPanel() ); 
		setFileNamePanel(new FileNamePanel(getStatusBar() ));
	}
	
	protected void setupGridLayout() {
		GridBagConstraints c = new GridBagConstraints();
		
		c.gridx = 0; 
		c.gridy = 0;
		
		getMainPanel().add(buttonPanel, c);
		
		// Filename Panel
		c.gridx = 0; 
		c.gridy = 1; 
		
		getMainPanel().add(getFileNamePanel(), c);					
		
		// VIEW button
		c.gridx = 0; 
		c.gridy = 2; 
		getMainPanel().add(getViewEditButton(), c);


		// SPACE BETWEEN components AND STATUS BAR //
		c.gridx = 0;
		c.gridy = 3;

		c.weightx  = 1.0;
		c.weighty  = 0.0;

		c.gridwidth = GridBagConstraints.REMAINDER;
		c.fill      	 = GridBagConstraints.BOTH;

		// put some vertical space
		getMainPanel().add(Box.createRigidArea(new Dimension(0, 20)), c);

		// STATUS BAR //    
		c.gridx = 0;
		c.gridy = 4;

		c.weightx = 1.0;
		c.weighty = 0.0;

		c.gridwidth = GridBagConstraints.REMAINDER;
		c.fill      	 = GridBagConstraints.HORIZONTAL;
		c.anchor    = GridBagConstraints.PAGE_END;

		getMainPanel().add(getStatusBar(), c);
	}
	
	protected void setupComponents() {
		getMainPanel().setLayout(new GridBagLayout() );
		
		getButtonGroup().add(getFirstInputRadio() );
		getButtonGroup().add(getSecondInputRadio() );
		getButtonGroup().add(getOutputRadio() ); 
		getButtonGroup().add(getFileNameRadio() );
		
		buttonPanel = new JPanel(new GridLayout(0, 1) );
		buttonPanel.add(getFirstInputRadio() );
		buttonPanel.add(Box.createRigidArea(new Dimension(0, 1) ));
		buttonPanel.add(getSecondInputRadio() );
		buttonPanel.add(Box.createRigidArea(new Dimension(0, 1) ));
		buttonPanel.add(getOutputRadio() );
		buttonPanel.add(Box.createRigidArea(new Dimension(0, 1) ));
		buttonPanel.add(getFileNameRadio() ); 
		
		// setup event handling for the buttons 
		getViewEditButton().addActionListener(this);
	}
	
	protected void addComponents() {
		setupGridLayout();
		setContentPane(getMainPanel() ); 
	}

	// writes the panels text into a temp file, and returns the name of the file	
	protected String handleInputPanels() {
		IOPanel sourcePanel; 
		boolean writeFile;  
	
		// determine which panel to obtain input					
		if ( this.getFirstInputRadio().isSelected() )
			sourcePanel = getRightPanel().getFirstInputPanel(); 
		else if ( this.getSecondInputRadio().isSelected() )
			sourcePanel = getRightPanel().getSecondInputPanel();
		else
			sourcePanel = getRightPanel().getOutputPanel();
	
		// write text to a temp file (used by command execution)				
		writeFile = sourcePanel.writeTextToFile();
							
		// unsuccessful file write
		if ( !writeFile ) {
			System.out.println("ViewDiagDialog: actionPerformed..problem writing to file");

			// don't proceed any further			
			return null;
		}		
		
		// otherwise we return the name of the file
		return sourcePanel.getTempFilePath(); 
	}
	


	// reads input file's into a string and calls Global's lookslikeFM method 
	protected boolean looksLikeFM(String inputFile) {
		try {
			if ( inputFile == null)
				return false; 
				
			BufferedReader reader = new BufferedReader(new FileReader(new File(inputFile) ));
			String text, line; 
			
			text = Global.EMPTY_STRING;
			line = reader.readLine();
			
			if ( line != null )
				text = line;
			
			while ( line != null ) {
				line = reader.readLine();
				
				if ( line != null)
					text += "\n" + line;
			}

			
			// with the file input read in as a string, see if it looks like an FM
			return Global.looksLikeFM(text);
		}
		
		// rare occasions... return false in this instance
		catch (Exception e) {
			return false;
		}
	}


	// display an information message when the input doesnt look like an FM
	private void showNotAnFMDialog() {
		String FM = "(START) |- 0 \n 0 a 1 \n 1 + 2 \n 2 b 3 \n 3 -| (FINAL)";
		
		String message = "The input does not appear to be a finite machine.  A finite machine should have at least one start state, and a " +
									"final state.  An example finite machine appears like this: \n" + FM;
									
		JOptionPane.showMessageDialog(null, message, "Attention!", JOptionPane.INFORMATION_MESSAGE);									
	}

	public void actionPerformed(ActionEvent ae) {
		String inputFile;
		
		// File radio button is selected
		if ( this.getFileNameRadio().isSelected() ) { 
			inputFile = this.getFileNamePanel().getValidFileName(JFileChooser.FILES_ONLY);
			
			// if user hasn't entered a valid filename, don't proceed any further
			if ( inputFile == null )
				return; 
		}

		// Input1, Input2, or Output radio button is selected
		else { 
			inputFile = handleInputPanels();
		}
		
		// if input doesnt look like an FM, dont proceed further
		if ( !looksLikeFM(inputFile) ) {
				if (D) System.out.println("actionPerformed: doesnt look like FM...");

			// prompt the user
			showNotAnFMDialog();
			
			// assign input file to null, so that View/EditDiagDialog do not proceed further
			setInputFile(null);
			
			return;
		}

		// now assign the input file (used by subclasses ViewDiagDialog and EditDiagDialog)
		setInputFile(inputFile);			   	
	} 
	
	
	// only enable a radio button if its corresponding textArea has data
	protected void setRadioStatus(JTextArea textArea, JRadioButton radioButton) {
		if ( textArea.getText().length() == 0 )
			radioButton.setEnabled(false);
			
		else
			radioButton.setEnabled(true);						
	}

	// only show options if their associated panels contain input 
	public void show() {
		// only enable radio buttons if their corresponding textAreas have data
		setRadioStatus(getRightPanel().getFirstInputPanel().getTextArea(), getFirstInputRadio() );
		setRadioStatus(getRightPanel().getSecondInputPanel().getTextArea(), getSecondInputRadio() );
		setRadioStatus(getRightPanel().getOutputPanel().getTextArea(), getOutputRadio() );
		
		// assign the selected radio button, depending on which one is enabled 
		if ( getFirstInputRadio().isEnabled() ) getFirstInputRadio().setSelected(true);
		else if ( getSecondInputRadio().isEnabled() ) getSecondInputRadio().setSelected(true);
		else if ( getOutputRadio().isEnabled() ) getOutputRadio().setSelected(true);
		else getFileNameRadio().setSelected(true);
			
		// now show the dialog
		super.show(); 
	}

	
	// GETTERS and SETTERS 
	protected void setViewEditButton(JButton viewEditButton) { this.viewEditButton = viewEditButton; }
	protected JButton getViewEditButton() { return this.viewEditButton; }	
	
	// radio button widgets
	protected void setFirstInputRadio(JRadioButton firstInputRadio) { this.firstInputRadio = firstInputRadio; }
	protected JRadioButton getFirstInputRadio() { return this.firstInputRadio; }	

	protected void setSecondInputRadio(JRadioButton secondInputRadio) { this.secondInputRadio = secondInputRadio; }
	protected JRadioButton getSecondInputRadio() { return this.secondInputRadio; }	

	protected void setOutputRadio(JRadioButton outputRadio) { this.outputRadio = outputRadio; }
	protected JRadioButton getOutputRadio() { return this.outputRadio; }	
	
	protected void setButtonGroup(ButtonGroup buttonGroup) { this.buttonGroup = buttonGroup; }
	protected ButtonGroup getButtonGroup() { return this.buttonGroup; }

	// widgets for inputting filename
	protected void setFileNameRadio(JRadioButton fileNameRadio) { this.fileNameRadio = fileNameRadio; }
	protected JRadioButton getFileNameRadio() { return this.fileNameRadio; }	

	// panels
	protected void setMainPanel(JPanel mainPanel) { this.mainPanel = mainPanel; }
	protected JPanel getMainPanel() { return this.mainPanel; }

	protected void setFileNamePanel(FileNamePanel fileNamePanel) { this.fileNamePanel = fileNamePanel; }
	protected FileNamePanel getFileNamePanel() { return this.fileNamePanel; }
	
	protected void setStatusBar(StatusBar statusBar) { this.statusBar = statusBar; }
	protected StatusBar getStatusBar() { return this.statusBar; }
	
	// rightpanel from JGrailGui.java... needed for handling actionPerformed events 
	protected void setRightPanel(RightPanel rightPanel) { this.rightPanel = rightPanel; }
	protected RightPanel getRightPanel()                    { return this.rightPanel; }

	// used for actionPerformed methods in subclasses ViewDiagDialog and EditDiagDialog
	protected void setInputFile(String inputFile) { this.inputFile = inputFile; }
	protected String getInputFile() 
	{ 
	System.out.println("getInputFile() ready to return ..." + this.inputFile);
	if (this.inputFile.charAt(inputFile.length()-1)==Global.UNIX_SLASH)
		{
		System.out.println("Last character should not be a UNIX slash");
		System.out.println("Therefore, we are returning: " + this.inputFile.substring(0, this.inputFile.length() -1 ) );
		return this.inputFile.substring(0, this.inputFile.length());
		}
	else 
	{
	System.out.println("inputFile is valid");
		return this.inputFile;
	}
	}
	
	

	public static void main(String[] args) {
		JFrame frmTest = new JFrame();
		frmTest.setSize(800, 600);

	   DiagDialog dd = new DiagDialog(frmTest, null, "View");
	   dd.show();
	}

}
