//Copyright 2005 Aaron Cooper
/*
This file is part of JGrail.
JGrail is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

JGrail is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with JGrail; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
*/

import javax.swing.*;
import javax.swing.text.*;

import java.awt.*; 
import java.awt.event.*;

import java.io.*;

/** 
* Represents the menu bar that appears on the main JGrail screen.
*
* @author Aaron Cooper
* @version 1.0 May-June 2004
*/ 

public class GrailMenuBar extends JMenuBar implements ActionListener {
	/** The index location where the item "Save First Input to File" appears under the File menu */ 
  protected static final int SAVE_FIRST_INPUT_INDEX  = 0;
	/** The index location where the item "Save Second Input to File" appears under the File menu */ 	
  protected static final int SAVE_SECOND_INPUT_INDEX = 1;
	/** The index location where the item "Save Output to File" appears under the File menu */ 	
  protected static final int SAVE_OUTPUT_INDEX       = 2;
	/** The index location where the item "Exit" appears under the File menu */ 	
  protected static final int EXIT_INDEX              = 3;

  /** The index location where the item "View as Diagram" appears under the Tools menu */ 	
  protected static final int VIEW_DIAG_INDEX = 0;
  /** The index location where the item "Edit as Diagram" appears under the Tools menu */ 	
  protected static final int EDIT_DIAG_INDEX  = 1;

	/** The index location where the item "How do I..." appears under the Help menu */ 	
  protected static final int HELP_FILE_INDEX         = 0;
	/** The index location where the item "About" appears under the Help menu */ 		
  protected static final int HELP_ABOUT_INDEX        = 1;

  private JMenu fileMenu, editMenu, confMenu, toolMenu, helpMenu; 
  private JGrailGui parentFrame;
  private ConfFile  confFile;  

	
	/** 
	* Creates a menu bar with a reference to its parent and access to program settings via the configuration file.
	*
	* @param parent the menu bar's parent (i.e the main JGrail screen)
	* @param file the configuration file which stores program settings 
	*/ 
  public GrailMenuBar(JGrailGui parent, ConfFile file) {
    createComponents();
    setupComponents();
    addComponents();

	// keep a reference to the parent frame this menu bar is in (used in the event handling below)
	setParentFrame(parent);
    setConfFile(file);  
  }

	
  // GETTERS & SETTERS //
  protected void setFileMenu(JMenu fileMenu) { this.fileMenu = fileMenu; }
  protected JMenu getFileMenu() { return this.fileMenu; }

  protected void setEditMenu(JMenu editMenu) { this.editMenu = editMenu; }
  protected JMenu getEditMenu() { return this.editMenu; }

  protected void setConfMenu(JMenu confMenu) { this.confMenu = confMenu; }
  protected JMenu getConfMenu() { return this.confMenu; }

  public void setToolMenu(JMenu toolMenu) { this.toolMenu = toolMenu; }
  public JMenu getToolMenu() { return this.toolMenu; }

  public void setHelpMenu(JMenu helpMenu) { this.helpMenu = helpMenu; }
  public JMenu getHelpMenu() { return this.helpMenu; }



  protected void setParentFrame(JGrailGui parentFrame) { this.parentFrame = parentFrame; }
  protected JGrailGui getParentFrame() { return this.parentFrame; }

  protected void setConfFile(ConfFile confFile) { this.confFile = confFile; }
  protected ConfFile getConfFile() { return this.confFile; }
    

  /**
  * Creates the GUI widgets for this menu bar. 
  */ 
  protected void createComponents() {
    setFileMenu(new JMenu("File") );
    setEditMenu(new JMenu("Edit") );
    setConfMenu(new JMenu("Configure") );
    setToolMenu(new JMenu("Tools") );
    setHelpMenu(new JMenu("Help") );
  }

  /** 
  * Adds the GUI widgets to this menu bar. 
  */  	
  protected void addComponents() {
    add(getFileMenu() );
	add(getEditMenu() ); 
    add(getConfMenu() );
    add(getToolMenu() );
    add(getHelpMenu() );
  }

  /** 
  * Assigns values to properties of the GUI widgets contained in this menu bar. 
  */  	
  protected void setupComponents() {
    createFileMenu();
    createEditMenu();
    createConfMenu();
    createToolMenu();
    createHelpMenu();
  }

	
	/** 
	* Creates and sets up the properties of the File menu 
	*/ 
  protected void createFileMenu() {
    JMenuItem temp = new JMenuItem("Save First Input To File");
    getFileMenu().add(temp);
    temp.addActionListener(this);

    temp = new JMenuItem("Save Second Input To File");
    getFileMenu().add(temp);
    temp.addActionListener(this);
    
    temp = new JMenuItem("Save Output To File");
    getFileMenu().add(temp);
    temp.addActionListener(this);

    temp = new JMenuItem("Exit");
    getFileMenu().add(temp);
    temp.addActionListener(this);
  }

	
	/** 
	* Creates and sets up the properties of the Edit menu 
	*
	* Code was obtained from the following source: 
	* http://java.sun.com/docs/books/tutorial/uiswing/misc/dnd.html#cut
	*/ 
	protected void createEditMenu() { 
		JMenuItem temp = new JMenuItem(new DefaultEditorKit.CutAction() ); 		
    temp.setText("Cut");
    temp.setMnemonic(KeyEvent.VK_T);
    getEditMenu().add(temp);

    temp = new JMenuItem(new DefaultEditorKit.CopyAction() );
    temp.setText("Copy");
    temp.setMnemonic(KeyEvent.VK_C);
    getEditMenu().add(temp);
    
    temp = new JMenuItem(new DefaultEditorKit.PasteAction() );
    temp.setText("Paste");
    temp.setMnemonic(KeyEvent.VK_P);
    getEditMenu().add(temp);
  }
	
	
	/** 
	* Creates and sets up the properties of the Configure menu 
	*/ 
  protected void createConfMenu() {
    JMenuItem temp = new JMenuItem("Modify Program Settings");
    getConfMenu().add(temp);
    temp.addActionListener(this);
  }
	

  /** 
  * Creates and sets up the properties of the Tools menu 
  */ 
  protected void createToolMenu() {
  	JMenuItem temp = new JMenuItem("View Finite Machine as Diagram...");
	getToolMenu().add(temp);
	temp.addActionListener(this);

	temp = new JMenuItem("Edit Finite Machine as Diagram...");
	getToolMenu().add(temp);
	temp.addActionListener(this);
  }
	
	/** 
	* Creates and sets up the properties of the Help menu 
	*/ 
  protected void createHelpMenu() {
    JMenuItem temp = new JMenuItem("How do I...");
    getHelpMenu().add(temp);
    temp.addActionListener(this);

    temp = new JMenuItem("About JGrail");
    getHelpMenu().add(temp);
    temp.addActionListener(this);
  }
	

  /**
  * Handles the clicking events of the menus. 
  * <p>
  * 
  * @param ae the ActionEvent object provided by Java
  */
	public void actionPerformed(ActionEvent ae) {
		Object source = ae.getSource(); 

    if ( source instanceof JMenuItem ) {
      JMenuItem selected = (JMenuItem) source;

      if ( getFileMenu().isMenuComponent(selected) )
        handleFileMenu(selected);
      else if ( getConfMenu().isMenuComponent(selected) )
        handleConfMenu(selected);
      else if ( getHelpMenu().isMenuComponent(selected) )
        handleHelpMenu(selected);
	  else if ( getToolMenu().isMenuComponent(selected) )
	    handleToolMenu(selected);	          
      // otherwise must have been the Edit menu
      // so we don't need to do anything else (handled automatically)        
      else
        return;
    }
  }

	
	/** 
	* Event-handling for the File menu. 
	* 
	* @param selected the menu item that was clicked by the user
	*/ 
  protected void handleFileMenu(JMenuItem selected) {
    // close the application if they click on the Exit button
    if ( selected == getFileMenu().getItem(EXIT_INDEX) ) {
      System.exit(0);
    }
    else {
      handleSave(selected);
    }
  }

	
  /** 
	* Event-handling for the Configure menu. 
	* 
	* @param selected the menu item that was clicked by the user
	*/ 
  protected void handleConfMenu(JMenuItem selected) {
    ConfDialog dlg = new ConfDialog(getParentFrame(), getConfFile() );
    
    dlg.setModal(true);
    dlg.showDlg();
    dlg.dispose();
    
    getParentFrame().getLeftPanel().changeListData(getConfFile().getUnaryFuncs(), getConfFile().getBinaryFuncs() );
  }
  
  protected void handleToolMenu(JMenuItem selected) {
	DiagDialog dd; 
	
	// create the View Diag Dialog
	if ( selected == getToolMenu().getItem(VIEW_DIAG_INDEX) ) {
		dd = new ViewDiagDialog(getParentFrame(), getParentFrame().getRightPanel(), 
											getParentFrame().getConfFile() );
	}
	
	// otherwise create the Edit DIag Dialog
	else {
		dd = new EditDiagDialog(getParentFrame(), getParentFrame().getRightPanel(), 
											getParentFrame().getConfFile() );
	}

	// now show the appropriate dialog
	dd.setModal(true);
	dd.show(); 
	dd.dispose();
  }
	
	/** 
	* Event-handling for the Help menu. 
	* 
	* @param selected the menu item that was clicked by the user
	*/ 
  protected void handleHelpMenu(JMenuItem selected) {
    if ( selected == getHelpMenu().getItem(HELP_FILE_INDEX) ) {
      HelpFileFrame frame = new HelpFileFrame(getParentFrame(), ConfFile.CONF_LOCATION);       
      frame.show();
    }

    else {
      HelpAboutDialog dlg = new HelpAboutDialog(getParentFrame() );
      dlg.show();
    }
  }
  

 /** 
	* Event-handling for the save operations (under the File menu). 
	* 
	* @param selected the menu item that was clicked by the user
	*/ 
  protected void handleSave(JMenuItem selected) {
    String path         = getConfFile().getSavePath();
    StatusBar statusBar = getParentFrame().getStatusBar();
    
    JFileChooser fileChooser = new JFileChooser(path);
    int returnVal = fileChooser.showSaveDialog(this);

    if ( returnVal == JFileChooser.APPROVE_OPTION ) {
      File f = fileChooser.getSelectedFile();
        System.out.println("selected file = " + f.getAbsolutePath() );

      // ensure the parent directory is writeable...
      if ( f.getParentFile() != null && f.getParentFile().canWrite() ) {
          System.out.println("file_dir is writeable...");
        JTextArea taSource = null;

        if ( f.exists() && f.canWrite() )
            System.out.println("file exists and can be written to");
        else if ( f.exists() && !f.canWrite() )
          statusBar.showErrorMsg("The file " + f.getAbsolutePath() + " exists and cannot be written to");

        // get the source textarea from the corresponding panel
        if ( selected == getFileMenu().getItem(SAVE_FIRST_INPUT_INDEX) )
          taSource = getParentFrame().getRightPanel().getFirstInputPanel().getInputTextArea();
        else if ( selected == getFileMenu().getItem(SAVE_SECOND_INPUT_INDEX) )
          taSource = getParentFrame().getRightPanel().getSecondInputPanel().getInputTextArea();
        else if ( selected == getFileMenu().getItem(SAVE_OUTPUT_INDEX) )
          taSource = getParentFrame().getRightPanel().getOutputPanel().getOutputTextArea();
        else {
            System.out.println("No save button selected !??");
          return;
        }

        // attempt save operation
        try {
          String fileName = f.getAbsolutePath();
          BufferedWriter writer = new BufferedWriter(new FileWriter(fileName) );

          taSource.write(writer);
          statusBar.showSaveMsg(Global.SAVE_FILE_MSG + fileName);
        }
        catch (Exception e) {
          statusBar.showErrorMsg("Unknown error has happened " + e.getMessage() );
        }
      } // if parent dir is writeable

      // otherwise we can't write in the directory....
      else {
        statusBar.showErrorMsg("Could not write in the location: " + f.getParentFile().getAbsolutePath() );
      }
        
    } // if APPROVE_OPTION
  }
}
