//Copyright 2005 Aaron Cooper
/*
This file is part of JGrail.
JGrail is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

JGrail is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with JGrail; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
*/

import javax.swing.*;
import javax.swing.event.*;

import java.awt.*; 
import java.awt.event.*;

import java.net.*;
import java.io.*;
import java.util.*;

import javax.swing.text.html.*;
import javax.swing.text.*;

/**
* Creates a frame that displays an HTML file.  Uses a "Frequently Asked Questions" type of format. 
* 
* @author Aaron Cooper
* @version 1.0 June 2004 
*/ 

class HelpFileFrame extends JFrame implements HyperlinkListener {
	/** The protocol to use to display the help file (i.e. not http or ftp) */ 
  protected static final String HELP_FILE_SCHEME = "file://";
	/** The name of the help file */ 
  protected static final String HELP_FILE        = "help.html";

	/** The default size for the frame */ 
  protected static final int    DFLT_FRAME_SIZE = 800;
	/** The title for the frame */ 
  protected static final String FRAME_TITLE     = "JGrail Help";

  private JEditorPane editPane;
  private JScrollPane editScrPane;
	private String helpFilePath; 
	

	/** 
	* Creates the help file frame.
	* 
	* @param owner the parent/owner for this frame
	* @param helpFilePath the location where the help file can be found
	*/ 
	public HelpFileFrame(JFrame owner, String helpFilePath) {
    super(FRAME_TITLE);

    setSize(owner); 
	setHelpFilePath(helpFilePath); 

    getContentPane().setLayout(new BoxLayout(getContentPane(), BoxLayout.PAGE_AXIS) );
   
    createComponents();
    setupComponents();
    addComponents();
	}
  

  protected void setSize(JFrame owner) {
    if ( owner == null )
      setSize(DFLT_FRAME_SIZE, DFLT_FRAME_SIZE);
    else
      setSize(owner.getSize().width - 30, owner.getSize().height - 30);
  }
  
  protected void setEditPane(JEditorPane editPane) { this.editPane = editPane; }
  protected JEditorPane getEditPane() { return this.editPane; }

  protected void setEditScrPane(JScrollPane editScrPane) { this.editScrPane = editScrPane; }
  protected JScrollPane getEditScrPane() { return this.editScrPane; }
	
	protected void setHelpFilePath(String helpFilePath) { this.helpFilePath = helpFilePath; }
	protected String getHelpFilePath() { return this.helpFilePath; }
    

	/**
	* Responds to hyperlinks being clicked in the help file.
	* 
	* @param e the HyperlinkEvent object provided by Java 
	*/ 
  public void hyperlinkUpdate(HyperlinkEvent e) {
    if ( e.getEventType() == HyperlinkEvent.EventType.ACTIVATED ) {
      JEditorPane pane = (JEditorPane) e.getSource();

      try {
        URL url = e.getURL();

        // for references on the help page (e.g. named anchors)
        if ( url != null && isHelpFile(url) ) {
          pane.scrollToReference(url.getRef() );
        }

        // for external references (i.e. on the web)
        else {
          pane.setPage(e.getURL());
        }
      }
      catch (Throwable t) {
        t.printStackTrace();
      }
    }
  }
  

	/** 
	* Determines if the given url is equal to the help file name.  Used to determine whether or not to load 
	* an external reference in the event handling routine (see hyperlinkUpdate).
	*
	* @param url the url that is to be checked for equivalence to the help file 
	* @return whether or not the argument is equal to the help file name
	*/
  protected boolean isHelpFile(URL url) {
    if ( url == null )
      return false;
    
    String urlPath = url.getPath();

      //System.out.println("isHelpFile...url.getPath() = " + url.getPath() );
      //System.out.println("isHelpFile...PATH + FILE   = " + getHelpFilePath() + HELP_FILE);

    // if the help file path and name are part of this url, then we must be linking inside the help file
    if ( urlPath.equals(getHelpFilePath() + Global.UNIX_SLASH + HELP_FILE) ) 
      return true;

    // assume unsuccessful
    return false;
  }
  

  /**
  * Creates the GUI widgets for this frame. 
  */ 
  protected void createComponents() {
    setEditPane(new JEditorPane() );

    //Put the editor pane in a scroll pane.
    setEditScrPane(new JScrollPane(getEditPane() ));
  }

  
	/** 
	* Creates and sets the properties of the JEditPane which holds the HTML file.
	*/ 
  protected void setupEditPane() {
    URL helpURL = null;

    getEditPane().setEditable(false);
    getEditPane().addHyperlinkListener(this);

    try {
      helpURL = new URL(HELP_FILE_SCHEME + getHelpFilePath() + Global.UNIX_SLASH + HELP_FILE);
      getEditPane().setPage(helpURL);
    }
    catch (MalformedURLException mue) {
      System.err.println("malformed url exception");
    }
    catch (IOException e) {
      System.err.println("Attempted to read a bad URL: " + helpURL);
    }
  }

	
  /** 
  * Assigns values to properties of the GUI widgets contained in this frame.
  */  	
  protected void setupComponents() {
    setupEditPane();
    
    getEditScrPane().setPreferredSize(new Dimension(250, 145));
    getEditScrPane().setMinimumSize(new Dimension(10, 10));
    
    getEditScrPane().setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    getEditScrPane().setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);    
  }


  /** 
  * Adds the GUI widgets to this frame. 
  */  	
  protected void addComponents() {
    getContentPane().add(getEditScrPane() );
  }

	
 /** 
  * Main function to test out functionality for this component.
  * 
  * @param args the array of arguments, provided by Java.
  */  
  public static void main(String[] args) throws Exception {
    HelpFileFrame frame = new HelpFileFrame(null, "/home/aaron/.jgrail/");
    frame.show();
  }
}
