//Copyright 2005 Aaron Cooper
/*
This file is part of JGrail.
JGrail is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

JGrail is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with JGrail; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
*/

import javax.swing.*;
import javax.swing.text.*;

import java.awt.*;
import java.awt.event.*;


/**
 * Popup menu that is shown when the user right-clicks on the input or output panels.  
 * <p>
 * 
 * @author Aaron Cooper
 * @version 1.0 June 4, 2005
 */

public class RightClickMenu extends JPopupMenu implements ActionListener {
	protected static final int CUT_INDEX    = 0; 
	protected static final int COPY_INDEX  = 1; 
	protected static final int PASTE_INDEX = 2; 
	
	protected static final int VIEW_INDEX = 4; // separator bar is at index 3 
	protected static final int EDIT_INDEX = 5; 
	
	// debug variable
	private boolean D = false;

	private JGrailGui parentFrame;
	private ConfFile confFile; 
	
	public RightClickMenu(Component[] listeners, JGrailGui parentFrame) {
		setupMenuItems();
		setupListeners(listeners);
		
		// keep a reference to the parent frame (used in the event handling below)
		setParentFrame(parentFrame);

		// used to obtain the viewer, editor, and other external tools 
		setConfFile(parentFrame.getConfFile() );
	}

	private void setupMenuItems() {
		JMenuItem temp;		

		// create the Cut, Copy and Paste items, for more info see: 
		// http://java.sun.com/docs/books/tutorial/uiswing/misc/dnd.html#cut  	
		temp = new JMenuItem(new DefaultEditorKit.CutAction() );
		temp.setText("Cut");
		temp.setMnemonic(KeyEvent.VK_T);
		this.add(temp);
		
		temp = new JMenuItem(new DefaultEditorKit.CopyAction() );
		temp.setText("Copy");
		temp.setMnemonic(KeyEvent.VK_C);
		this.add(temp);

		temp = new JMenuItem(new DefaultEditorKit.PasteAction() );
		temp.setText("Paste");
		temp.setMnemonic(KeyEvent.VK_P);
		this.add(temp);
		
	
		// group the cut, copy and paste commands
		this.addSeparator();

		temp = new JMenuItem("View Finite Machine as Diagram...");
		this.add(temp);
		temp.addActionListener(this);

		temp = new JMenuItem("Edit Finite Machine as Diagram...");
		this.add(temp);
		temp.addActionListener(this);
	}
	
	// register all components that will "listen" and "show" the popup menu
	// e.g. a text-area will show the popup, but a textfield would not
	private void setupListeners(Component[] listeners) {
		MouseListener m = new PopupListener(this);
		
		for (int i=0; i < listeners.length; i++)
			listeners[i].addMouseListener(m);
	}


	// handle the case when popup is shown over filename textfields
	protected String handleFileTextField(Function externalTool, Component invoker) {
		JTextField sourceTxtFld;
		String inputFile;
		FileNamePanel parentPanel;  
		
		if ( invoker == getParentFrame().getRightPanel().getFirstInputPanel().getFileNamePanel().getFileNameTxtFld() ) {
			sourceTxtFld = getParentFrame().getRightPanel().getFirstInputPanel().getFileNamePanel().getFileNameTxtFld();
		}
		else {
			sourceTxtFld = getParentFrame().getRightPanel().getSecondInputPanel().getFileNamePanel().getFileNameTxtFld();
		}
		
		 // get the textfield's parent panel (either firstInputPanel or secondInputPanel) 
		 parentPanel = (FileNamePanel) sourceTxtFld.getParent();
		 
		 // grab the filename
		 inputFile = parentPanel.getValidFileName(JFileChooser.FILES_ONLY);
		 
		 // return the file name
		 return inputFile;
	}
	
	// handle the case when the popup is shown over the textareas 
	protected String handleTextArea(Function externalTool, Component invoker) {
		IOPanel sourcePanel;
		boolean writeFile;
		
		// determine the panel in which the RightClickMenu has been shown 
		sourcePanel = getInvokerPanel(invoker); 
		
		// this case shouldn't happen
		if (sourcePanel == null)
			return null; 
			
		// write text to a temp file (used by command execution)				
		writeFile = sourcePanel.writeTextToFile();
						
		// unsuccessful file write
		if ( !writeFile ) {
			System.out.println("handleTextArea...problem writing to file");

			// don't proceed any further			
			return null;
		}		
		
		// otherwise we return the name of the temp file
		return sourcePanel.getTempFilePath();
	}

	
	// determine if popup occurred due to textfield or text area, and redirect to above methods
	protected String handleEvent(Function externalTool, JMenuItem source) {
		if ( this.getInvoker() instanceof JTextField ) 
			return handleFileTextField(externalTool, this.getInvoker() );
		else if ( this.getInvoker() instanceof JTextArea )
			return handleTextArea(externalTool, this.getInvoker() );
		else 
			return null; // case shouldn't happen		
	}
	
	// show the external tool (either the viewer or editor)
	protected void showTool(Function externalTool, String inputFile) {
		if ( externalTool == getEditor() )
			// convert the file to the editor's file type 	
			inputFile = convert(inputFile);

		// set the input file
		externalTool.addInFile(inputFile);
		
			System.out.println("showTool() - inputFile = " + inputFile);
			
			
		// execute the command
		externalTool.execAsThread(); 
	}
	
	// converts the inputfile into the editor's file type
	protected String convert(String inputFile) {
		boolean execCmd; 
		String outFile; 
		
		// if no convertor exists, we'll use the inputfile as is
		if ( getFmToEditor() == null )
			return inputFile; 
		
		// name of the output file for the conversion program
		outFile = inputFile + ".conv";
	
		getFmToEditor().addInFile(inputFile);
		getFmToEditor().setOutFile(outFile); 

		// try conversion		
		try {
			execCmd = getFmToEditor().execNoErrorHandling();
		}
		// if a rare exception occurs, we'll set boolean value to false
		catch (Exception e) {
			execCmd = false;
		}

		// if command failed, we'll try running the editor on the inputfile
		if ( !execCmd ) {
				System.out.println("returning inputFile");
			return inputFile;
		}

		// otherwise we'll run the editor on the converted file (i.e. output of the conversion program)
		else {
			System.out.println("returning outFile");
			return outFile;
		}  			
	}


	// determines if the input appears to be an FM 
	private boolean looksLikeFM(IOPanel sourcePanel) {
		if ( sourcePanel != null ) {
			String text = sourcePanel.getTextArea().getText();  

			if ( Global.looksLikeFM(text) )
				return true; 
		}
			
		return false;
	}

	public void actionPerformed(ActionEvent ae) {
		String inputFile;
		Function externalTool;
		JMenuItem source = (JMenuItem) ae.getSource();
		
			if (D) System.out.println("rightClickMenu: actionPerformed");
		
		if ( source == this.getComponent(VIEW_INDEX) ) {
			inputFile = handleEvent(this.getViewer(), source); 
			
			 if ( inputFile == null )
				return; 
			
			// now show the program
			showTool(this.getViewer(), inputFile);
		}

		else if ( source == this.getComponent(EDIT_INDEX) ) {
			inputFile = handleEvent(this.getEditor(), source); 
			
			if ( inputFile == null )
				return; 
				
				if (D) System.out.println("rightClickMenu:actionPerformed(): inputFile = " + inputFile);
			
			// now show the program
			showTool(this.getEditor(), inputFile);
		}
		
		else 
			return; 
	}
	
	
	// only show the "View as Diagram" and "Edit as Diagram" options if the input text looks like an FM
	public void show(Component invoker, int x, int y) {
		IOPanel sourcePanel; 
			if (D) System.out.println("right click show....");

		sourcePanel = this.getInvokerPanel(invoker); 			
			
		// right now, the right click menu only appears on the IOPanels on the main JGrail screen... if this changes, implement appropriate functilnality here
		if (  sourcePanel == null ) {
				if (D) System.out.println("invoker isn't an IOPanel...");
			super.show(invoker, x, y);
			
			return;			  
		}
		
			if (D) System.out.println("rightclick show... going to see if it looks like an FM");
			
		// enable the View/Edit options if it looks like an FM
		if ( looksLikeFM(sourcePanel) ) {
			this.getComponent(VIEW_INDEX).setEnabled(true);
			this.getComponent(EDIT_INDEX).setEnabled(true);			
		}
		
		// otherwise disable them
		else {
			this.getComponent(VIEW_INDEX).setEnabled(false);
			this.getComponent(EDIT_INDEX).setEnabled(false);			
		}
		
		// now show the right click menu
		super.show(invoker, x, y);
	}
	
	
	// internal class that will handle the showing of the popup menu
	class PopupListener extends MouseAdapter {
		JPopupMenu popupMenu; 
		
		public PopupListener(JPopupMenu popupMenu) {
			this.popupMenu = popupMenu;
		}

		// handle both mousePressed and mouseReleased, as explained in isPopupTrigger() 
		// see http://java.sun.com/j2se/1.4.2/docs/api/java/awt/event/MouseEvent.html
		public void mousePressed(MouseEvent e) {maybeShowPopup(e);}
		public void mouseReleased(MouseEvent e) {maybeShowPopup(e);}

		// only show the popup if the "trigger" button was pressed
		private void maybeShowPopup(MouseEvent e) {
			if (e.isPopupTrigger()) {
				popupMenu.show(e.getComponent(),
						   e.getX(), e.getY());
			}
		}
	}
	
	
	// find out the source panel in which the rightclick menu is displayed
	protected IOPanel getInvokerPanel(Component invoker) {
		IOPanel sourcePanel;
			
			System.out.println("getInvokerPanel....invoker info: " + invoker.getClass().getName() );
			
		if ( invoker == getParentFrame().getRightPanel().getFirstInputPanel().getTextArea() ) 
			sourcePanel = getParentFrame().getRightPanel().getFirstInputPanel();
		else if ( invoker == getParentFrame().getRightPanel().getSecondInputPanel().getTextArea() ) 
			sourcePanel = getParentFrame().getRightPanel().getSecondInputPanel();
		else if ( invoker == getParentFrame().getRightPanel().getOutputPanel().getTextArea() )
			sourcePanel = getParentFrame().getRightPanel().getOutputPanel(); 
		else
			sourcePanel = null; 
			
		return sourcePanel;  				
	}
	
	protected void setParentFrame(JGrailGui parentFrame) { this.parentFrame = parentFrame; }
	protected JGrailGui getParentFrame() { return this.parentFrame; }
	
	protected ConfFile getConfFile() { return this.confFile; }
	protected void setConfFile(ConfFile confFile) { this.confFile = confFile; }
	
	protected Function getViewer() 		{return getConfFile().getViewer(); }
	protected Function getEditor() 		{return getConfFile().getEditor(); }   
	protected Function getFmToEditor() {return getConfFile().getFmToEditor(); }
	


	public static void main(String[] args) {
		JFrame frame = new JFrame("tester");
		Component[] listeners = {frame}; 
		
		
		frame.setSize(500, 500);
		
		RightClickMenu menu = new RightClickMenu(listeners, null); 
		
		frame.show();
	}
}
