//Copyright 2005 Aaron Cooper
/*
This file is part of JGrail.
JGrail is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

JGrail is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with JGrail; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
*/

import java.io.*;
import java.util.*; 

/** 
* Represents the JGrail Configuration File.  It stores the values it reads from the configuration file, so that they 
* can be accessed by other JGrail components.  The configuration file stores its values in a "key=value" format.
*
* @author Aaron Cooper
* @version 1.0 May-June 2004
*/ 

class ConfFile {
	/** The name of the configuration file */ 
	public static final String CONF_FILE        = "jgrail.conf";
	/** The name of the backup configuration file */ 
	public static final String BAK_CONF_FILE = CONF_FILE + ".old";
	/** The folder name in which the configuration file is located (not the path, just the name of the folder, e.g. "jgrail/") */ 
	public static final String CONF_DIR = "/.jgrail";
	/** The folder name where the Grail+ commands are located */
	public static final String GRAIL_EXEC_DIR = "";
	
	/** directory where configuration files are stored (eg /home/aaron/.jgrail) */	
	public static final String CONF_LOCATION = Global.USER_HOME_LOCATION + CONF_DIR;
	/** directory full path and the name of the conf file */ 
	public static final String CONF_FULL_PATH  = CONF_LOCATION + Global.UNIX_SLASH + CONF_FILE; 
	/** directory full path for the temporary directory (eg /home/aaron/.jgrail/tmp) */ 
	public static final String TMP_LOCATION = CONF_LOCATION + Global.UNIX_SLASH + "tmp"; 

  
	/** The name of the section header in which Unary Functions are defined in the configuration file */ 
	protected static final String UNARY_HEADER  = "***Unary Functions***"; 
	/** The name of the section header in which Binary Functions are defined in the configuration file */ 
	protected static final String BINARY_HEADER = "***Binary Functions***"; 
	/** The name of the section header in which Directory paths are defined in the configuration file */ 
	protected static final String DIR_HEADER    = "***Directories***"; 
	/** The name of the section header in which Tools are defined in the configuration file */ 
	protected static final String TOOLS_HEADER    = "***Tools***"; 

  
  /** The name of the key which defines the path where the Grail+ executables are located */ 
  protected static final String EXEC_PATH_KEY = "EXEC_PATH";
  /** The name of the key which defines the path where the user saves his output */ 
  protected static final String SAVE_PATH_KEY = "SAVE_PATH";
  
  /** The name of the key which defines the name of the Grail+ command */
  protected static final String NAME_KEY       = "name";
  /** The name of the key which defines the command string that executes the Grail+ commands */  
  protected static final String CMD_KEY       = "cmd";
  /** The name of the key which defines the description of the Grail+ commands */ 
  protected static final String DESC_KEY      = "desc";
  /** The name of the key and value pair that defines that the Grail+ command takes a literal string as an argument */  
  protected static final String STRING_ARG_KEY_VALUE_PAIR = "opt=$STRING";

  
  
  /** The keys used to define the name of the external Tools */ 
  protected static final String[] TOOL_KEYS = {"viewer", "editor", "fm_to_editor", "reg_exp_to_fm", "finite_lang_to_fm" };

  // indices in the tools array 
  protected static final int VIEWER_INDEX 						  = 0; 	
  protected static final int EDITOR_INDEX						  = 1;
  protected static final int FM_TO_EDITOR_INDEX 			  = 2;
  protected static final int REG_EXP_TO_EDITOR_INDEX 	  = 3;
  protected static final int FINITE_LANG_TO_EDITOR_INDEX = 4;

  // number of paths expected under the DIR_HEADER section in the conf file
  protected static final int NUM_PATHS  = 2;
    	
  protected static final String PARSE_ERR_MSG = "There was a problem parsing the configuration file.  You may wish to manually investigate " +
  																			"this or alternatively, run JGrailSetup again";	

  // debug variable	
  protected static final boolean D = true; 
  
  //// INSTANCE VARIABLES  
  private Vector  unaryFuncs;  
  private Vector  binaryFuncs;
  private String  execPath, savePath; 
  private Function[] tools;   // an array of external programs (eg xfig)	

  private String  saveErrorMsg;
  private int lineNum; // used when parsing the conf file

  /**
  * Creates an object representing the configuration file.  Sets up the directory which houses the configuration file, but read it.  
  * To read the configuration file, see parseConfFile().
  * <p>
  * Assumes the configuration file is located in the user directory, under the CONF_FILE_FOLDER directory..
  * e.g. if CONF_FILE_FOLDER is "jgrail", and user directory for a UNIX system is "/home/joe", then the location would be /home/joe/jgrail/"
  */ 
  public ConfFile() {
    // initialize instance vars
    setUnaryFuncsVector(null);
    setBinaryFuncsVector(null);
    setExecPath(null);
    setSavePath(null);
  }

	
  /** 
  * Saves changes to the configuration file.  Performs error checking to make sure the configuration file exists.  
  * 
  * @param unaryFuncs the vector that stores unary Grail+ functions
  * @param binaryFuncs the vector that stores binary Grail+ functions
  * @param execPath the location where the Grail+ executables are located 
  * @param savePath the location where the user stores his output
  * @param tools an array the external programs (eg. xfig)
  * 
  * @return whether the save operation was successful or not
  */ 
  public boolean saveChanges(Vector unaryFuncs, Vector binaryFuncs, String execPath, String savePath, Function[] tools) {
    String filePath = CONF_FULL_PATH;

    File f = new File(filePath);
    File parentDir = f.getParentFile();

    // start off with no error messages
    setSaveErrorMsg(null);

      System.out.println("abs path = " + f.getAbsolutePath() );

    if ( !parentDir.exists() ) {
      setSaveErrorMsg("Directory does not exist - " + parentDir.getAbsolutePath() );
    }
    
    else if ( !parentDir.canWrite() ) {
      setSaveErrorMsg("Can't Write into parent directory - " + parentDir.getAbsolutePath() );
    }

    else if ( !f.exists() ) {
      setSaveErrorMsg("Couldn't find the file " + filePath);
    }

    else {
      boolean backup = f.renameTo(new File(filePath + ".old") );

      if ( !backup ) {
        setSaveErrorMsg("Couldn't back up " + filePath + " to " + filePath + ".old");
      }

      else
        return doChanges(unaryFuncs, binaryFuncs, execPath, savePath, tools);
    }

    // assume unsuccessful saving...
    return false;
  }


	  /** 
  * Saves the unary and binary function values to the configuration file.  
  * <p>
  * The "functions Vector" is a Vector of Vectors.  This is because the ConfDialog Window displays the 
  * unary/binary functions to the user in a JTable.  To get the rows/cols of the JTable, the getTableVector() function 
  * was used.  Therefore, the Vector of Vectors represents the rows/cols of the JTable.  For more information, please 
  * refer to TableModel.getTableVector()
  * 
  * @param sectionHeader the section to write the values under in the conf file (e.g. Unary or Binary)
  * @param writer the BufferedWriter used to write to the conf file
  * @param functions a Vector of Vectors which contains the values for the unary/binary functions
  * 
  * @return a vector of unary/binary functions
  */ 
  private Vector saveFuncs(String sectionHeader, BufferedWriter writer, Vector functions) {
    try {
      String name, desc;
      Boolean takesStringArg;
      Vector vRow, vFuncs;

      vFuncs = new Vector();
      
      writer.write(sectionHeader, 0, sectionHeader.length() );
      writer.newLine();

      for (int row = 0; row < functions.size(); row++) {
        vRow = (Vector) functions.elementAt(row);

        name = (String) vRow.elementAt(Global.FIRST_COL);
        desc  = (String) vRow.elementAt(Global.SECOND_COL);
		takesStringArg = (Boolean) vRow.elementAt(Global.THIRD_COL);

		  if (D) {	
		  	System.out.println("name = " + name);
          	System.out.println("desc = " + desc);
 		  	System.out.println("takesStringArg = "  + takesStringArg);
		  }

		writer.write(NAME_KEY + Global.EQUAL_SIGN + name, 0, NAME_KEY.length() + Global.EQUAL_SIGN.length() + name.length() );
		writer.newLine();
		writer.write(DESC_KEY + Global.EQUAL_SIGN + desc, 0, DESC_KEY.length() + Global.EQUAL_SIGN.length() + desc.length() );
		writer.newLine();
		
		// if the Grail+ function takes a literal string argument, then write it as an option in the conf file
		// otherwise we will avoid it and go on to the next Grail+ function to write
		if ( takesStringArg.booleanValue() == true ) {
        	writer.write(STRING_ARG_KEY_VALUE_PAIR, 0, STRING_ARG_KEY_VALUE_PAIR.length() );
        	writer.newLine();
		}
		        	
        writer.newLine();

        // add new function to vector
        vFuncs.addElement(new Function(name, desc, null, getExecPath() ));
      }

      // return the vector of functions
      return vFuncs;
    }
    catch (Exception e) {
      setSaveErrorMsg("Error happened when writing function data to file " + e.getMessage() );
      return null;
    }
  }
  
  
  private Function[] saveTools(String sectionHeader, BufferedWriter writer, Function[] tools) {
	try {
	  String name, cmd, desc;
	  
	  // write the section header
	  writer.write(sectionHeader, 0, sectionHeader.length() );
	  writer.newLine();
	  
	  // write the cmd and descriptions to the file
	  for (int i=0; i < tools.length; i++) {		
		name = tools[i].getName();
	  	desc   = tools[i].getDesc();
		cmd   = tools[i].getCmd();
		
		if (D) {	
		  System.out.println("name = " + name);
		  System.out.println("desc = " + desc);
		  System.out.println("cmd = " + cmd);		
		}

	
		writer.write(TOOL_KEYS[i]  + Global.EQUAL_SIGN + name,  0, TOOL_KEYS[i].length()  + Global.EQUAL_SIGN.length() + name.length() );
		writer.newLine();
		writer.write(DESC_KEY + Global.EQUAL_SIGN + desc, 0, DESC_KEY.length() + Global.EQUAL_SIGN.length() + desc.length() );
		writer.newLine();
		writer.write(CMD_KEY + Global.EQUAL_SIGN + cmd, 0, CMD_KEY.length() + Global.EQUAL_SIGN.length() + cmd.length() );
		writer.newLine();
		writer.newLine();
	  }

	  // now return the array of tools		  	
	  return tools;		  	
	}
	
	// handle exceptions
	catch (Exception e) {
		setSaveErrorMsg("Error happened when writing tools data to file " + e.getMessage() );
		return null;
	}
  }

	
  /** 
  * Saves the EXEC and SAVE paths to the conf file.  
  * 
  * @param dirHeader the section under which the path values should be saved in the conf file (e.g. "Directories")
  * @param execPath the location where the Grail+ executables are located 
  * @param savePath the location where the user stores his output
  */ 
  private void saveDirs(String dirHeader, BufferedWriter writer, String execPath, String savePath) {
    try {
      writer.write(dirHeader, 0, dirHeader.length() );
      writer.newLine();

      writer.write(EXEC_PATH_KEY + Global.EQUAL_SIGN + execPath, 0, EXEC_PATH_KEY.length() + Global.EQUAL_SIGN.length() + execPath.length() );
      writer.newLine();

      writer.write(SAVE_PATH_KEY + Global.EQUAL_SIGN + savePath, 0, SAVE_PATH_KEY.length() + Global.EQUAL_SIGN.length() + savePath.length() );
      writer.newLine();
      writer.newLine();
    }
    catch (IOException e) {
      setSaveErrorMsg("Error Writing To File..." + e.getMessage() );
    }
  }

  
  /** 
  * Writes the changes to the conf file.  Calls the functions saveDirs() and saveFuncs().  
  * <p>
  * The vector arguments are actually "vectors of vectors".  For more info, please see the note on saveFuncs(). 
  * 
  * @param unaryFuncs the vector that stores unary Grail+ functions
  * @param binaryFuncs the vector that stores binary Grail+ functions
  * @param execPath the location where the Grail+ executables are located 
  * @param savePath the location where the user stores his output
  * @param tools an array the external programs (eg. xfig)
  * 
  * @return whether or not the operation was successful 
  */ 
  private boolean doChanges(Vector unaryFuncs, Vector binaryFuncs, String execPath, String savePath, Function[] tools) {
    File f = new File(CONF_FULL_PATH);

    BufferedWriter writer = null;
    Vector v;
    String cmd, desc;
        
    try {
      writer = new BufferedWriter(new FileWriter(f) );

      // add terminating directory slash if needed....
      execPath = Global.addSlash(execPath);
      savePath = Global.addSlash(savePath);
      
      // now we save the directories (to conf file)
      saveDirs(DIR_HEADER, writer, execPath, savePath);
      setExecPath(execPath);
      setSavePath(savePath);

      // something bad happened when saving dirs
      if ( getSaveErrorMsg() != null )
        return false;

      setUnaryFuncsVector(saveFuncs(UNARY_HEADER, writer, unaryFuncs) );
      // something bad happened when saving unary funcs
      if ( getSaveErrorMsg() != null )
        return false;
        
      setBinaryFuncsVector(saveFuncs(BINARY_HEADER, writer, binaryFuncs) );
      // something bad happened when saving binary funcs
      if ( getSaveErrorMsg() != null )
        return false;
     
     
	    if (D) {
			System.out.println("before saving....");
		     for (int i=0; i < tools.length; i++) {
				if ( tools[i] == null )
					System.out.println("tools[" + i + "] is null");
				else {
					System.out.println("tools[" + i + "] cmd =" + tools[i].getName() );
					System.out.println("tools[" + i + "] desc =" + tools[i].getDesc() );
					System.out.println("tools[" + i + "] cmd =" + tools[i].getCmd() );
					System.out.println("tools[" + i + "] path =" + tools[i].getExecPath() );
				}
		     }
	    }
     
   	 setToolsArray(saveTools(TOOLS_HEADER, writer, tools) );

	 /*
	 System.out.println("after saving....");   	 
	 for (int i=0; i < tools.length; i++) {
		System.out.println("tools[] cmd =" + tools[i].getCmd() );
		System.out.println("tools[] desc =" + tools[i].getDesc() );
		System.out.println("tools[] path =" + tools[i].getPath() );
	 }
	 */

	  // something bad happened when saving binary funcs
	  if ( getSaveErrorMsg() != null )
		return false;
      
      // otherwise if we get here, then successful saving...
      return true;
    }

    catch (Exception e) {
      setSaveErrorMsg("Unknown error occurred " + e.getMessage() );
      return false;
    }

    finally {
    	// ensure writer is closed	
    	try {
    		writer.close();
    	}    		
    	catch (IOException io) {
    		Global.showBugErrorDialog(io); 
    	}
    }
    
  }


 /** 
  * Converts a vector into a Function array.  More info can be obtained at 
  * http://forum.java.sun.com/thread.jsp?forum=31&thread=200420
  * 
  * @return an array of Functions
  */ 
  private Function[] convert(Vector v) {
    Function[] f = new Function[v.size() ];

    return (Function[]) v.toArray(f);
  }



  /**
  * Parses the configuration file.  Calls parseSection(). 
  */ 
  public void parseConfFile() throws Exception {
    // now parse the sections
		if (D) { System.out.println("\t\t going to parse Unary");} 
    parseSection(UNARY_HEADER);  

    	if (D) { System.out.println("\t\t going to parse Binary");} 
    parseSection(BINARY_HEADER);  

		if (D) { System.out.println("\t\t going to parse Dir Header");} 
    parseSection(DIR_HEADER);  

		if (D) { System.out.println("\t\t going to parse the Tools");} 
	parseSection(TOOLS_HEADER);  
      //print();
  }

	
  /** 
  * Parses a section of the conf file.  Populates the unary/binary vectors, as well as the EXEC and SAVE paths. 
  * Calls parsePaths() and parseSectionValues() 
  * 
  * @param sectionHeader the section header under which the values will be parsed (e.g. "Unary") 
  */ 
  private void parseSection(String sectionHeader) throws Exception {
    BufferedReader reader = null;
    try {
      reader = new BufferedReader(new FileReader(CONF_FULL_PATH) );
    }
    catch (FileNotFoundException fne) {
     // if you cant find the file make one
    	// added 11/16/08 Max Ahlquist
    	makeDefaultConfigFile makeConfFile=new makeDefaultConfigFile();
    	makeConfFile.createConfig();
    	
    	// couldn't find the conf file....
      //throw new ConfFileException("Couldn't find the file " + CONF_FULL_PATH);
    }

	// read first line
    String line = reader.readLine();

	// initial line number that has been parsed    								
	setLineNum(1);    								
    
    // skip all lines until we hit the Section Header
    while ( line != null && !line.equals(sectionHeader) ) {
      line = reader.readLine();
      incLineNum();
        if (D) { System.out.println("skipping lines until section found... line = " + line); }  
    }

    // couldn't find section....so we dont proceed further
    if ( line == null ) {
	  throw new ConfFileException("Section \" " + sectionHeader + " \" was not found"); 
    }
    // parse the PATH variables
    else if ( sectionHeader.equals(DIR_HEADER) ) {
      parsePaths(reader); 
    }
    // parse the function values
    else if ( sectionHeader.equals(UNARY_HEADER) ) {
      setUnaryFuncsVector(parseFunctionValues(reader) );
    }
    else if (sectionHeader.equals(BINARY_HEADER) ) {
      setBinaryFuncsVector(parseFunctionValues(reader) );
    }
    else {
    	parseToolValues(reader);
    }
  }
	
  /**
  * Parses the EXEC_PATH and SAVE_PATH variables in the conf file 
  * 
  * @param reader the BufferedReader to read values from the conf file 
  */ 
  private void parsePaths(BufferedReader reader) throws Exception {
    int numParsedPaths = 0; 
    
    // go past section header
    String line = readNextValidLine(reader);
	
    // same looping construct as in parseSectionValues()
    while ( line != null && !isSectHeader(line) ) {
      if ( !isComment(line) && !Global.isEmptyLine(line) ) {
        // if the line doesn't contain the EXEC_PATH key, try parsing the SAVE_PATH
        if ( line.indexOf(EXEC_PATH_KEY) == Global.NOT_FOUND ) {
			setSavePath(Global.addSlash(parseKeyValuePair(line, SAVE_PATH_KEY) ));
			numParsedPaths++;
        }
          

        // otherwise we'll try to parse the EXEC_PATH          
        else {
			setExecPath(Global.addSlash(parseKeyValuePair(line, EXEC_PATH_KEY) ));
			numParsedPaths++;
        }
          
      }
      
      // get the next line
	  line = readNextValidLine(reader);
    }
    
    // execpath is mandatory, but was not found
    if ( getExecPath() == null ) { 
    	throw new ConfFileException("EXEC_PATH was not found, but it is mandatory");  
    }
  }
  
  
  /** parses the tools from the conf file */
  private void parseToolValues(BufferedReader reader) throws Exception {
	   String line; 
	   String cmd, desc;
	   Function t;  
    
	   Function[] tools = new Function[Global.NUM_TOOLS];
	   
	   int numParsedTools = 0; 

	   // go past section header  
	   line = readNextValidLine(reader);
    
	   // read all values until EOF or the next section header is reached
	   while ( line != null && !isSectHeader(line) ) {
		   // parse the function
		   t = parseTool(reader, line, TOOL_KEYS[numParsedTools] ); 

		   // add the parsed function to the vector 
		   tools[numParsedTools] = t; 
		   
		   numParsedTools++; 

		   // get next line      
		   line = readNextValidLine(reader); 
	   }
	
		// set remaining tool array to null, if need be
	   	for (int i=numParsedTools; i < Global.NUM_TOOLS; i++) {
			tools[i] = null;  	         
	   	}

	   // now assign the tools array
  	  setToolsArray(tools);
  }  	

	
  /**
  * Parses the values under the Unary/Binary headers in the conf file
  * 
  * @param reader the BufferedReader to read values from the conf file 
  * @param lineNum the current line in which the BufferedReader is currently pointing to 
  * 
  * @return a vector of parsed functions (unary or binary)
  */ 
  private Vector parseFunctionValues(BufferedReader reader) throws Exception {
	String line; 
	Function f; 
    
	Vector v = new Vector();

	// go past section header  
	line = readNextValidLine(reader);
    
	// read all values until EOF or the next section header is reached
	while ( line != null && !isSectHeader(line) ) {
		// parse the function
		f = parseFunction(reader, line, NAME_KEY); 

		// add the parsed function to the vector (only if it's not null)
		if ( f != null )
 			v.addElement(f); 

	  	// get next line      
	  	line = readNextValidLine(reader); 
	  	
	  		if (D) { 
	  			if ( line != null ) {
	  				System.out.println("about to check for opt=$STRING... readNextValidLine returned the following:"); 
	  				System.out.println(line);
	  			}	  					  		
	  		}
	  		
	  	// check for the special key/value pair that defines a Grail+ command that takes a literal string as an argument
	  	// (this is a very rare case)
	  	if ( line != null && line.indexOf(STRING_ARG_KEY_VALUE_PAIR) != Global.NOT_FOUND && f != null) {
	  		f.setTakesStringArg(true); 
	  		
	  		// since we found the rare option, read in the next line  
	  		line = readNextValidLine(reader);
	  	}
	}
	
	// return vector of parsed functions
	return v;
  }
  
  
 /** parses one key=value pair in the conf file  */ 	
 private String parseKeyValuePair(String line, String key) throws Exception {
   String upLine, upKey;
   String value;
   int index; 
   	 
   // uppercase 
   upLine = line.trim().toUpperCase(); 
   upKey = key.toUpperCase();  
	 
   // couldn't find the key 
   if ( line.indexOf(key) == Global.NOT_FOUND ) {
	  throw new ConfFileException("error at line " + getLineNum() + ": expecting value \"" + key + "\", but found value \"" + line + "\""); 
   }
	 
   // parse the equal sign
   index = line.indexOf(Global.EQUAL_SIGN);
      
   // equal sign not found      
   if ( index == Global.NOT_FOUND ) { 
	  throw new ConfFileException("error at line " + getLineNum() + ": expecting equal sign after key \"" + key + "\"");
   }
	
   
   // otherwise we parsed it OK, so return the value 
	// if no value given, use the empty string
	if ( index + 1 >= line.length() )
		value = Global.EMPTY_STRING; 
	// otherwise parse the value		
	else 
	    value = line.substring(index + 1);
   
      if (D) { System.out.println("just parsed the following..." + key + " = " + value); } 
   
   return value; 
 }
  

  
  /** parses one Function from the confFile */ 
  private Function parseFunction(BufferedReader reader, String line, String key) throws Exception {
	 String name, cmd, desc; 

	 // parse the name	 
	 name = parseKeyValuePair(line, key); 
	 
	 // read in the next line
	 line = readNextValidLine(reader); 	 

	 // unexpectedly reached EOF
	 if ( line == null ) {
		throw new ConfFileException("line " + getLineNum() + ": expecting value \"" + DESC_KEY + "\", but found the end of the file");  
	 }
	 
	 // parse the corresponding description key	
	 desc = parseKeyValuePair(line, DESC_KEY); 

 	 // Name and Desc are mandatory for Grail+ commands 
 	 // if either is missing, we won't include it
 	 if ( Global.isEmptyLine(name) || Global.isEmptyLine(desc) )
 		 return null; 
 	 else 
	   // return parsed Function        
	   return new Function(name, desc, null, getExecPath() );              
  } 

  /** parses one Tool (ie the viewer, editor, and converters) from the confFile */ 
  private Function parseTool(BufferedReader reader, String line, String key) throws Exception {
	 String name, cmd, desc; 

	 // parse the name	 
	 name = parseKeyValuePair(line, key); 
	 
	 // read in the next line
	 line = readNextValidLine(reader); 	 

	 // unexpectedly reached EOF
	 if ( line == null ) {
		throw new ConfFileException("line " + getLineNum() + ": expecting value \"" + DESC_KEY + "\", but found the end of the file");  
	 }
	 
	 // parse the corresponding description key	
	 desc = parseKeyValuePair(line, DESC_KEY); 

	 // read in the next line
	 line = readNextValidLine(reader); 	 

	 // unexpectedly reached EOF
	 if ( line == null ) {
	   throw new ConfFileException("line " + getLineNum() + ": expecting value \"" + CMD_KEY + "\", but found the end of the file");  
	 }
	 
	 //	parse the corresponding command key	
	 cmd = parseKeyValuePair(line, CMD_KEY); 
 
	 // return null if the Command OR the Name are empty 
	 // if missing, we won't include this Tool
	 if ( Global.isEmptyLine(cmd) || Global.isEmptyLine(name) )
		 return null; 
	 else 
	   
	 // return parsed Function        
	 return new Function(name, desc, cmd, getExecPath() );              
  } 


  /** skips over comments and empty lines, returning a valid line from the conf file */
  private String readNextValidLine(BufferedReader reader) throws Exception {
	 String line;
  	 
	 line = reader.readLine(); 
	 incLineNum(); 
  	 
		//System.out.println("\t\t\t readNextValidLine(): line = " + line);
  	 
	 // skip over lines while it is a comment, or an empty line		
	 while ( line != null && (isComment(line) || Global.isEmptyLine(line) )) {
		line = reader.readLine(); 
		incLineNum();
		
			//System.out.println("\t\t\t readNextValidLine(): line = " + line);
	 }
  	 
	 return line;
  }
  

  /** 
  * Returns the unary functions vector as an array 
  * 
  * @return an array of Function objects
  */ 
  public Function[] getUnaryFuncs() {
    if ( getUnaryFuncsVector() == null ) {
      Function[] f = {null};
      return f;
    }

    return convert(getUnaryFuncsVector() );
  }

  
  /** 
  * Returns the binary functions vector as an array 
  * 
  * @return an array of Function objects
  */ 
	public Function[] getBinaryFuncs() {
    if ( getBinaryFuncsVector() == null ) {
      Function[] f = {null};
      return f;
    }

    return convert(getBinaryFuncsVector() );
  }  

	
  /** 
  * Determines if any values were parsed from the config file
  * 
  * @return boolean value, representing whether or not any values were parsed from the conf file
  */ 
  public boolean isEmpty() {
    if ( getUnaryFuncsVector().size() == 0 && getBinaryFuncsVector().size() == 0 && getExecPath() == null && getSavePath() == null )
      return true;
    return false;
  }
	
	
  /** 
  * Determines whether or not the given argument is "commented-out"
  * 
  * @param line the line to check for comments
  * @return boolean value, representing whether or not the given argument is "commented-out"
  */ 
  private boolean isComment(String line) {
    if ( line == null || line.length() == 0 )
      return false;

    return line.trim().charAt(0) == Global.COMMENT_CHAR;
  }

	
  /** 
  * Determines if the given argument represents a section header in the conf file 
  * 
  * @param line the line to check for a section header
  * @return boolean value, representing whether or not the line is a section header 
  */ 
  private boolean isSectHeader(String line) {
    return ( line.equals(UNARY_HEADER) || line.equals(BINARY_HEADER) || line.equals(DIR_HEADER) || line.equals(TOOLS_HEADER) );
  }
  

  // AGAINST THE STANDARD -- should be called "getUnaryFuncs()"... but getUnaryFuncs() already defined above for another purpose
  protected void   setUnaryFuncsVector(Vector unaryFuncs) { this.unaryFuncs = unaryFuncs; }
  protected Vector getUnaryFuncsVector() { return this.unaryFuncs; }
  
	// AGAINST THE STANDARD -- same reason as above //
  protected void setBinaryFuncsVector(Vector binaryFuncs) { this.binaryFuncs = binaryFuncs; }
  protected Vector getBinaryFuncsVector() { return this.binaryFuncs; }

  protected void setSaveErrorMsg(String saveErrorMsg) { this.saveErrorMsg = saveErrorMsg; }
  public  String getSaveErrorMsg() { return this.saveErrorMsg; } // PUBLIC //

  protected void setExecPath(String execPath) {
    if ( execPath != null )
      this.execPath = execPath.trim();
    else
	  this.execPath = execPath;
  }
  public  String getExecPath() { return this.execPath; } // PUBLIC //

  protected void setSavePath(String savePath) {
    if ( savePath != null )
      this.savePath = savePath.trim();
    else
	this.savePath = savePath;
  }
  public  String getSavePath() { return this.savePath; } // PUBLIC //

  public String getConfFileName() { return CONF_FILE; } // PUBLIC //

  // used when parsing the conf file
  private void incLineNum() { this.lineNum++; }
  private void setLineNum(int val) { this.lineNum = val; }
  private int getLineNum() { return this.lineNum; }


  protected void setToolsArray(Function[] tools ) { this.tools = tools; }
  protected Function[] getToolsArray() { return this.tools; }
  
  	// temporary...
  	/*
	protected void setEditor(Tool editor) { tools[EDITOR_INDEX] = editor; }
  	protected void setViewer(Tool viewer) { tools[VIEWER_INDEX] = viewer; }
	protected void setFmToEditor(Tool fmToEditor) { tools[FM_TO_EDITOR_INDEX] = fmToEditor; }
	*/
	
  public Function getEditor() { return getToolsArray()[EDITOR_INDEX]; }	
  public Function getViewer() { return getToolsArray()[VIEWER_INDEX]; }  
  public Function getFmToEditor() { return getToolsArray()[FM_TO_EDITOR_INDEX]; }
  public Function getRegExpToEditor() { return getToolsArray()[REG_EXP_TO_EDITOR_INDEX]; }
  public Function getFiniteLangToEditor() { return getToolsArray()[FINITE_LANG_TO_EDITOR_INDEX]; }




   /* DEBUGGING FUNCTIONS */
	
  /**
   * Prints the values contained in the vector.  Used for debugging. 
   * 
   * @param v the vector which contains values to be printed 
   */ 
   private void printVector(Vector v) {
	 if ( v != null ) {
		 Enumeration e = v.elements(); 
		 Function f;
		 
		 while (e.hasMoreElements() ) {
		 	f = (Function) e.nextElement();
		 	
		 	if ( f == null )
		 		System.out.println("printVector(): f is null");
		 	else 
		 		f.print();
		 }
	 }
   }

   /** 
   * Prints the values contained in the array.  Used for debugging. 
   * 
   * @param f the array which contains values to be printed
   */ 
   private void printArray(Function[] f) {
	 for (int i=0; i < f.length; i++)
		if ( f[i] == null )
			System.out.println("f[" + i + "] = " + "null" );
		else { 	   	
			System.out.print("f[" + i + "] = "); 
			f[i].print();
		}
	   		
   }
  

   /** 
   * Prints the values of the unary and binary vectors, as well as the path variable values.  Used for debugging.
   */ 
   private void print() {
	 printVector(getUnaryFuncsVector() );
	 printVector(getBinaryFuncsVector() );
	 System.out.println("EXEC_PATH = " + getExecPath() );
	 System.out.println("SAVE_PATH = " + getSavePath() );
	 printArray(getToolsArray() );
   }

  /** 
  * Main function to test out functionality for this component.
  * 
  * @param args the array of arguments, provided by Java.
  */  
  public static void main(String[] args) throws Exception {
    ConfFile confFile = new ConfFile(); 

    confFile.parseConfFile();
    confFile.print();
  }

}
