//Copyright 2005 Aaron Cooper
/*
This file is part of JGrail.
JGrail is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

JGrail is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with JGrail; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
*/
import java.util.Vector; 
import java.util.Enumeration;
import javax.swing.JOptionPane;

import java.io.*;

/**
 * An external command, that knows how to execute itself (eg xfig, fmtofig, and the Grail+ programs)
 * 
 * @author Aaron Cooper
 * @version 1.0 28-May-2005
 */
public class Function {
	/** The name of the shell to execute the commands */ 
	protected static final String SHELL = "sh";
  
	/** Arguments passed to the shell that will execute commands */ 
	protected static final String SHELL_OPTS = "-c";
	
	/** Number of milliseconds in a second */  
	protected static final int SECOND 				= 1000; 
	
	/** Delay time waiting for a process to complete execution */
	protected static final int WAIT_DELAY 		= 1 * SECOND / 2; 
	
	/** Counter for loop that waits for a process to complete execution */ 
	protected static final int WAIT_NUM_TIMES = 10;

	/** Number that represents an unfinished process' exit value */
	protected static final int NOT_DONE = -99; 
	
	/** If the program hangs, error output is written to this file */ 
	protected static final String ERROR_FILE = ConfFile.CONF_LOCATION + Global.UNIX_SLASH + "jgrail.error";

	/** Max number of threads this Function will hold a reference to... */
	protected static final int MAX_THREADS = 5; 

	protected String name, desc, cmd; 

	protected String outFile;
	protected Vector inFiles; 
	protected String stringArg;
	
	// used when executing external programs (e.g. xfig)
	// not necessary to hold on to references to the threads, but just to be safe, we'll store them in an array
	// see execTool() for more info
	protected ExecProgThread[] threads;
	int numThreads;
	
	/** Certain Grail+ commands take a literal string as an arg, as opposed to a file */
	protected boolean takesStringArg; 
	
	/** location of this Function (eg /usr/local/jgrail/bin) */ 
	protected String execPath;  

	public Function(String name, String desc, String cmd, String execPath) {
		setName(name);
		setDesc(desc);
		setCmd(cmd);
		setExecPath(execPath);
		
		setInFiles(new Vector() );
	}

	// executes the command associated with this Tool
	// internal function.. outside classes invoke the exec() command
	protected boolean runCommand() throws Exception {
		String completeCmd = buildCommandString();
		
		int exitVal = NOT_DONE;
		
			System.out.println("runCommand: completeCmd = " + completeCmd);
		
		// command needs to be run with a shell...
		//http://java.sun.com/developer/JDCTechTips/2003/tt0304.html#1      
		String[] args = {SHELL, SHELL_OPTS, completeCmd};
                
		// run the command
		Runtime rt   = Runtime.getRuntime();
		Process proc = rt.exec(args);
	
		// query the process' exit value until either it finishes, or we reach the counter max
		for (int i=0; i < WAIT_NUM_TIMES; i++) {
			try {
					System.out.println("going to sleep");
				Thread.sleep(WAIT_DELAY);
				   System.out.println("return from sleep");
				exitVal = proc.exitValue();
					
				// if exitVal is given a value, it means the process finished
				// so we quit the loop
				break; 
			}
			
			catch (IllegalStateException ise) {
					System.out.println("runCommand(): illegal state exception caught...");
				exitVal = NOT_DONE;
			}
		}
			
		// if the process isn't done yet, then it must be hanging
		// destroy it and return false, signalling unsuccessful execution
		if ( exitVal == NOT_DONE ) {
				System.out.println("process is hanging... going to destroy");
                                
			// stop the process
			proc.destroy(); 
                        
			// return unsuccessful operation
			return false; 
		}
	
		System.out.println("terminated with exit code = " + exitVal); 
		
		  
		// if we get here, must have been successful command execution			  
		return true;
	}


	// this should write the erorr output to a file... to be done later
	private void writeErrorOutputToFile(String cmd) throws Exception {
		JOptionPane.showMessageDialog(null, ErrorMessages.HANGING_PROGRAM);		
	}
	
	
	
	// executes the function, but doesn't display any dialogs if an error occurs
	// assumes error handling taken care of outside the function
	public boolean execNoErrorHandling() throws Exception {
		return runCommand();
	}
        
  
	// executes the function and handles errors by presenting appropriate dialogs to the user
	public void exec() {
		try {
			boolean notHanging = runCommand();
			
			if ( !notHanging) 
				writeErrorOutputToFile(this.getCmd() );
		}
		
		// handle rare cases with the bug dialog		
		catch (Exception e) {
				System.out.println("exec(): caught Exception e = " + e.getMessage() );
				System.out.println("class = " + e.getClass() );
			Global.showBugErrorDialog(e);				
		}
	}      

	public void execAsThread() {
		String completeCmd = buildCommandString();
		
		System.out.println("completeCmd = " + completeCmd);
		 // command needs to be run with a shell...
		 //http://java.sun.com/developer/JDCTechTips/2003/tt0304.html#1      
		 String[] args = {SHELL, SHELL_OPTS, completeCmd};
                              
		 ExecProgThread thread = new ExecProgThread(args);
		 addThreadToArray(thread);
		 thread.start(); 
	}
        

	//	replaces all $INFILE with its associated value from the Vector		
	protected String replaceInFiles(String cmdString) {
		Enumeration e = this.inFiles.elements();
		String inFile =""; 

		// replace all $INFILE with its associated value from the Vector (for binary functions)		
		while( cmdString.indexOf("$INFILE") != Global.NOT_FOUND && e.hasMoreElements() ) { 
			inFile = (String) e.nextElement();
		if (inFile.charAt(inFile.length()-1)==Global.UNIX_SLASH)
		{
		System.out.println("Last character of a file name should not be a UNIX slash");
		System.out.println("Therefore, we are returning: " + inFile.substring(0, inFile.length() -1 ) );
		inFile = inFile.substring(0, inFile.length()-1);
		}

			// must use double-slashes to escape the dollar-sign character
			// see class Pattern at http://java.sun.com/j2se/1.4.2/docs/api/ 
			cmdString = cmdString.replaceFirst("\\$INFILE", inFile);
		}
		
		// now return the command string
		System.out.println("raplceInFiles returned " + cmdString);
		return cmdString;

	}
	
	
	// replace occurrences of $INFILE and $OUTFILE in the command with their respective values
	protected String buildCommandString() {
		String newCmdString = getCmd();
			System.out.println("buildCommandString:...getCmd() just returned... " + getCmd() );
		
		// replace all occurrences of $INFILE
		newCmdString = replaceInFiles(newCmdString);
		
		// replace all occurrences of $OUTFILE
		if ( newCmdString.indexOf("$OUTFILE") != Global.NOT_FOUND ) {
			// must use double-slashes to escape the dollar-sign character (see above)
			newCmdString = newCmdString.replaceAll("\\$OUTFILE", getOutFile() );
		}
		
		// prepend all "command" occurrences with the path to it
		// e.g. "cat myfile.txt | myCommand" becomes "cat myfile.txt | /usr/bin/myCommand"
		if ( getName() != null ) {
			String commandFullPath; 
			
			// dont add a slash if the execpath already has one
			if ( getExecPath().charAt(getExecPath().length() - 1 ) == Global.UNIX_SLASH )
				commandFullPath = getExecPath() + getName(); 
			else 
				commandFullPath = getExecPath() + Global.UNIX_SLASH +getName(); 
			
			System.out.println("buildCommandString: going to replace the name with the full command path");
			
			// now replace the command will the full path to it	
			newCmdString = newCmdString.replaceAll(getName(), commandFullPath);  		
		}
		
		
		// purge the infiles (ensure they get populated on next invocation by corresponding addInFile() )	
		getInFiles().clear();
		
		// ditto with the outfile
		setOutFile(Global.EMPTY_STRING);			
		
		System.out.println("newCmdString = " + newCmdString);
		return newCmdString;  
	}

	// GETTERS and SETTERS
	public void addInFile(String inFile) {
		this.inFiles.addElement(inFile);
	}
	
	public void setInFiles(Vector v) {this.inFiles = v;}
	public Vector getInFiles() { return this.inFiles; }

	/** place the outfile in the /tmp directory under the use'rs home folder (e.g /home/aaron/.jgrail/tmp) */
	public void setOutFile(String outFile) {this.outFile = outFile;}
	public String getOutFile() { return this.outFile; }

	public void setStringArg(String stringArg) { this.stringArg = stringArg; }
	public String getStringArg() { return this.stringArg; }
	
	public void setName(String name) { this.name = name;}
	public void setCmd(String cmd) { this.cmd = cmd;}
	public void setDesc(String desc) { this.desc = desc;}	
	
	public String getName() { return this.name; }
	public String getCmd()  { return this.cmd;  }
	public String getDesc() { return this.desc; }
	
	public void setExecPath(String path) { this.execPath = path; }
	public String getExecPath() { return this.execPath; }
	
	// used to display value in the combo box
	public String toString() { return this.desc; }

	public boolean takesStringArg() { return takesStringArg; }
	public void setTakesStringArg(boolean value) { this.takesStringArg = value; }
	
	// hold on to reference to the executed thread
	private void addThreadToArray(ExecProgThread et) {
		if ( threads == null ) {
			threads = new ExecProgThread[MAX_THREADS];
			numThreads = 0;
	  	}
            
	  	threads[numThreads] = et; 
	  	
	  	// wrap around to first index of array... since by the time this occurs, the first thread will MOST likely ceased execution 
	  	// also, it's quite unlikely that the user will have MAX_THREAD instances of xfig running....
	  	numThreads = (numThreads + 1) % MAX_THREADS;            
	}
	
	// make sure the threads are destroyed when the user clicks the X button on the JGrailGui interface
	// see JGrailGui for more info
	public void destroyThreads() {
		/* AC Aug 11, 2005 -- not shutting down the xfig and fmtoview windows... to be determined
		if ( threads != null ) {
			for (int i=0; i < MAX_THREADS - 1; i++) {
				if ( threads[i] != null && threads[i].isAlive() ) {
						System.out.println("destroying threads...");
					//threads[i].interrupt();
					//threads[i].destroy();
					threads[i].setDone(true);
				}
			}
		}
		*/
	}

	
	public void print() {
		System.out.println("name = " + getName() );			
		System.out.println("desc = " + getDesc() );
		System.out.println("cmd = " + getCmd() );
	}

	public static void main(String[] args) {
		String name = ""; 
		String myString = "i'm a string";
		
		String result = myString.replaceAll(name, "test");
		
		System.out.println("name = " + name);
		System.out.println("result = " + result);
	}
}







  









