//Copyright 2005 Aaron Cooper
/*
This file is part of JGrail.
JGrail is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

JGrail is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with JGrail; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
*/

import java.awt.*; 
import java.awt.event.*; 
import javax.swing.*;

import java.io.*;  

/** 
* The main JGrail screen.  Contains left and right subpanels, as well as a menu bar and status bar.
*
* @author Aaron Cooper
* @version 1.0 May-June 2004 
*/ 

class JGrailGui extends JFrame implements ActionListener, WindowListener {
  /** The default starting height for the frame */ 
  protected static final int START_HEIGHT   = 800; 
  /** The default starting width for the frame */ 
  protected static final int START_WIDTH    = 800;

	/** The pixel-size of the invisible border */ 
  protected static final int EMPTY_BDR_SIZE = 20;
	/** The pixel-size of the invisible component placed between widgets */ 
  protected static final int EMPTY_CMP_SIZE = 15;

	/** The name of the shell to execute the Grail+ commands */ 
  protected static final String SHELL      = "sh";
	/** Arguments passed to the shell that will execute Grail+ commands */ 
  protected static final String SHELL_OPTS = "-c";

	/** Name of the first temp file name, used when performing Grail+ command executions */ 
  protected static final String TEMP_FIRST_INPUT_FULL_PATH = ConfFile.TMP_LOCATION + "/input1";
	/** Name of the second temp file name, used when performing Grail+ command executions */ 	
  protected static final String TEMP_SECOND_INPUT_FULL_PATH = ConfFile.TMP_LOCATION + "/input2";
	/** Name of the  temp output file name, used when performing Grail+ command executions */ 	
  protected static final String TEMP_OUTPUT_FULL_PATH = ConfFile.TMP_LOCATION + "/output";

	/** Title of the main screen */ 
  protected static final String FRAME_TITLE      = "JGrail - Graphical Interface for Grail+";

	// for debugging
	private boolean D = false; 

  private GrailMenuBar menuBar;
  private StatusBar   statusBar;

  private String computeErrorMsg;
  private String tempDirPath;

  private JPanel     mainPanel;
  private LeftPanel  leftPanel;
  private RightPanel rightPanel;
  
  // right-click context menu
  private JPopupMenu popupMenu;

  private ConfFile confFile;


	/** 
	* Creates the main screen, with program settings obtained from the configuration file 
	*/ 
  public JGrailGui(ConfFile confFile) {
    super();

    // set up frame values
    setTitle(FRAME_TITLE); 
    setSize(START_WIDTH, START_HEIGHT);
		
		Color p = new Color(255, 255, 204); 
		setBackground(p); 

    // instance vars
    setConfFile(confFile);
    
    // create the components
    createComponents();

    // add to content pane
    addComponents();
    
    // register window listener
    this.addWindowListener(this);
  }


  /** 
  * Assigns values to properties of a GridBagLayout manager. 
  * <p>
  * More information on how to use GridBagLayout can be obtained at 
  * http://java.sun.com/docs/books/tutorial/uiswing/layout/gridbag.html
  */ 
  private void setupGridLayout() {
    JPanel newContent    = new JPanel();
    GridBagConstraints c = new GridBagConstraints();

    newContent.setLayout(new GridBagLayout() );

    // MAIN PANEL //
    c.gridx = 0;
    c.gridy = 0;

    c.weightx  = 1.0;
    c.weighty  = 1.0;

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.fill      = GridBagConstraints.BOTH;

    newContent.add(getMainPanel(), c);

    // SPACE BETWEEN MAIN AND STATUS BAR //
    c.gridx = 0;
    c.gridy = 1;

    c.weightx  = 1.0;
    c.weighty  = 0.0;

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.fill      = GridBagConstraints.BOTH;

    // put some vertical space
    newContent.add(Box.createRigidArea(new Dimension(0, 20)), c);

    // STATUS BAR //    
    c.gridx = 0;
    c.gridy = 2;

    c.weightx = 1.0;
    c.weighty = 0.0;

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.fill      = GridBagConstraints.HORIZONTAL;
    c.anchor    = GridBagConstraints.PAGE_END;

    newContent.add(getStatusBar(), c);

	// make the content pane have the same background color as all the other sub-panels
	newContent.setBackground(Global.PEACH); 
		
    // re-assign the content pane
    setContentPane(newContent);
  }

	
  /** 
  * Adds the GUI widgets to this frame. 
  */  	
  private void addComponents() {
    getMainPanel().setLayout(new BoxLayout(getMainPanel(), BoxLayout.LINE_AXIS) );

    // add left and right, with empty space between them
    getMainPanel().add(getLeftPanel() );
    getMainPanel().add(Box.createRigidArea(new Dimension(EMPTY_CMP_SIZE, 0) ));
    getMainPanel().add(getRightPanel() );
		
		// make the main panel have the same background color as all the other sub-panels		
		// -- fills-in the vertical strip between the left and right panels (comment this line out to see what i mean)
		getMainPanel().setBackground(Global.PEACH); 

	//	put panMain on top, and a status bar on the bottom
	setupGridLayout();  	
    
    // add the menu bar to the frame
    setJMenuBar(getGrailMenuBar() );
  }
	
	
  /**
  * Creates the GUI widgets for this frame. 
  */ 
  private void createComponents() {
	// status bar on the bottom
	setStatusBar(new StatusBar() );

	// left and right (right panel depends on the status bar being created first...)
    setRightPanel(new RightPanel(ConfFile.TMP_LOCATION, this ));
    getRightPanel().setComputeButtonHandler(this);

    // panLeft depends on some objects from panRight... so create after...
    try {
      setLeftPanel(new LeftPanel(getRightPanel().getHideFirstChkBox(), getRightPanel().getHideSecondChkBox(),
                                 getConfFile().getUnaryFuncs(), getConfFile().getBinaryFuncs() ));
    }
    catch (Exception e) {
      System.out.println(e.getMessage() );
      e.printStackTrace();
    }   

    // main will contain left and right
    setMainPanel(new JPanel() );

    // menu bar on top
    setGrailMenuBar(new GrailMenuBar(this, getConfFile() ));

    
    // create the right-click context menu
    createRightClickMenu(); 
  }
  
  // create the right-click menu	
  private void createRightClickMenu() {
	Component[] listeners = new Component[5];  	
	
	// build an array of GUI widgets that will "listen" and "show" the popup menu
	listeners[0] = this.getRightPanel().getFirstInputPanel().getTextArea();
	listeners[1] = this.getRightPanel().getSecondInputPanel().getTextArea();
	listeners[2] = this.getRightPanel().getOutputPanel().getTextArea();
	
	// allows cut, copy and paste options to be visible on the filename textfields
	listeners[3] = this.getRightPanel().getFirstInputPanel().getFileNamePanel().getFileNameTxtFld();
	listeners[4] = this.getRightPanel().getSecondInputPanel().getFileNamePanel().getFileNameTxtFld();
	
	// now create it
	setPopupMenu(new RightClickMenu(listeners, this) );
  }
  
  
  // informs the user that they need to supply additional input 
  private void showMissingInputMsg() { 
	// if binary function being performed, tell them they need to supply input for both		 
	if ( getLeftPanel().getFuncsPanel().getBinaryRadio().isSelected() ) { 
		getStatusBar().showErrorMsg(ErrorMessages.NO_BINARY_INPUT);
	}

	// otherwise a unary function is missing data
	else {		 
		getStatusBar().showErrorMsg(ErrorMessages.NO_INPUT);
	}
  }


	// if a binary function is being performed, obtains its second argument
	private String getSecondArg(Function firstFunc) {
	   // if firstFunc is "fmexec" or "flexec", it takes a literal string as argument..
	   // i.e. can't use "fmexec file1 file2" syntax... so return a string as input 
	   if ( firstFunc.takesStringArg() ) {
	   		if (D) { System.out.println("func has a string arg");	}
		  return getInputAsString(getRightPanel().getSecondInputPanel() ); 
	   }
	   
	   // otherwise we return a filename as input
	   else {
			if (D) { System.out.println("func DOES NOT have a string arg");	}
		  return getInputAsFile(getRightPanel().getSecondInputPanel(), TEMP_SECOND_INPUT_FULL_PATH);
	   }
	}    


	// transfers the input from the textarea/textfield to a file and returns the name of the file
	// if a problem occurs, returns null      
	private String getInputAsFile(InputPanel panInput, String tempFilePath) {
	   JTextArea taSource        		= panInput.getInputTextArea();    
	   FileNamePanel panFileName = panInput.getFileNamePanel();
     
	   // write the textarea's input to a file
	   if ( taSource.getText() != null && taSource.getText().length() > 0 ) {
		try {
		  		if (D) { System.out.println("getInputAsFile: writing textarea data to file");	}
		  BufferedWriter writer = new BufferedWriter(new FileWriter(tempFilePath) );
		  taSource.write(writer);

		  // return the name of the temp file we just created
		  return tempFilePath;
		}

		// handle any exceptions (these are rare cases, so show a bug error dialog)		
		catch (Exception e) {
		  BugErrorDialog d = new BugErrorDialog(); 
	  	  d.addText(ErrorMessages.WRITE_TO_FILE + tempFilePath); 
	  	  d.setMessage(e); 
		}
	  }

	  // return the filename specified in the textfield
	  else if ( !panFileName.isEmpty() && !panFileName.isDefaultText() ) { 
			if (D) { System.out.println("getInputAsFile: grabbing filename from textfield");	}
		 return panFileName.getValidFileName(JFileChooser.FILES_ONLY);	
	  }

	  // otherwise there has been no input entered
	  else {
		 showMissingInputMsg();  
	  }	       
    
	  // if we get here, a problem must have occurred, so return null
	  return null; 
	}   
  

	// returns the textarea's text or transfers the input from the filename into a string, and returns this string
	// Grail+ functions, like fmexec, require one of their inputs to be a literal string
	// if a problem occurs, returns null      
	private String getInputAsString(InputPanel panInput) {
	   JTextArea taSource      		 = panInput.getInputTextArea();    
	   FileNamePanel panFileName = panInput.getFileNamePanel();

	   // return the textarea's text	
	   if ( taSource.getText() != null && taSource.getText().length() > 0 ) {
		 	 if (D) { System.out.println("getInputAsString: returning textarea text"); }
		  return taSource.getText();
	   }		          		
	
	   // read the data from the file specified by the textfield and return this data as a string	
	   else if ( !panFileName.isEmpty() && !panFileName.isDefaultText() ) { 		
		  String fileName = panFileName.getValidFileName(JFileChooser.FILES_ONLY); 
	  	  String input, line;
	  	  
				if (D) { System.out.println("getInputAsString: going to read from file"); } 
	  	  // initialize
	  	  input = ""; 
        
		  try {
			 BufferedReader reader = new BufferedReader(new FileReader(fileName) ); 
        
		 	 // get the first line
			 line = reader.readLine(); 
	
		 	 // read the file until EOF
			 while ( line != null ) {
				// append to string
				input += line; 
	   
				// read next line
				line = reader.readLine(); 
			 }

		 	 // return the input now
		 	 return input; 	
	  	}
 	    
 	    // handle any exceptions (these are rare cases, so show a bug error dialog)		
		catch (Exception e) {
			BugErrorDialog d = new BugErrorDialog(); 
			d.addText(ErrorMessages.READING_FILE + fileName); 
			d.setMessage(e); 
		}
	  }	
     
	  // otherwise there has been no input entered
	  else {
		 showMissingInputMsg(); 
	  }	       
    
	  // if we get here, a problem must have occurred, so return null
	  return null; 
	}   


  /**
  * Handles the clicking events of the Compute button. 
  * <p>
  * Determines if the temp directory exists, and then executes the Grail+ commands by running a shell. 
	* Displays the output in the right-hand side output panel.
  * 
  * @param ae the ActionEvent object provided by Java
  */
  public void actionPerformed(ActionEvent ae) {
	String firstInputFile, secondInputFile;
	JRadioButton rbBinary;
	ListModel listData;
	Function f;

	// get the first function to perform... obtained from the first drop down list
	String execPath    = getConfFile().getExecPath() + ConfFile.GRAIL_EXEC_DIR ; 
	JComboBox cb      = getLeftPanel().getFuncsPanel().getFuncCombo();
	Function firstFunc  = (Function)cb.getSelectedItem();
	String command   = execPath + firstFunc.getName();
	
		// DEBUG
		//f = new Function("test", "test desc", "cmd", null);
		//String execPath = "/usr/local/JGrail/bin";
		//Function firstFunc = new Function("firstFunc", "func desc", "firstFunc cmd", execPath);
		//String command = execPath + Global.UNIX_SLASH + "myFunc";		
	
	// get the name of the input file that will be used for the first func
	firstInputFile = getInputAsFile(getRightPanel().getFirstInputPanel(), TEMP_FIRST_INPUT_FULL_PATH);
	System.out.println("First Input File is :"+firstInputFile);
	  
	if ( firstInputFile != null ) {
            firstInputFile.trim();

        // if the PATH  end with a "/", ... then we delete it
        if ( firstInputFile.length() > 0 && firstInputFile.charAt(firstInputFile.length() - 1) == Global.UNIX_SLASH )
                            firstInputFile = firstInputFile.substring(0,firstInputFile.length()-1);
        }
	// if something has gone wrong when getting the inputfile, stop remaining execution
	if ( firstInputFile == null ) 
	  return; 
	  
	command += " " + firstInputFile;
	
	// get second input file, if the "Binary" radio button is clicked
	rbBinary = getLeftPanel().getFuncsPanel().getBinaryRadio();
	  
	// get second arg, if binary function being performed
	if ( rbBinary.isSelected() ) {
	   String secondArg = getSecondArg(firstFunc); 
	  
	   // if problem occurred, halt remaining execution
	   if ( secondArg == null ) 
	      return ;
		    
	   command += " " + secondArg; 
	}	 
	
	// build up the rest of the command, piping to the other programs obtained from the JList        
	listData = getLeftPanel().getListFuncsPanel().getFuncList().getModel();
	for (int i=0; i < listData.getSize(); i++) {
	  f = (Function) listData.getElementAt(i);
	  command += " " + Global.UNIX_PIPE + " " + execPath + f.getName();
	}
	
	// re-direct all command output to a temp file
	command += " " + Global.RE_DIRECT + " " + TEMP_OUTPUT_FULL_PATH; 
	
	  System.out.println("Grail+ command(s) to run = " + command);
	
	f = new Function(null, null, command, null); 
		   		
	// execute the command
	f.exec();
		
	// show output of the above command
	showResults(TEMP_OUTPUT_FULL_PATH);
  }
  

  // destroy all threads and shut down the app when user clicks X button
  public void windowClosing(WindowEvent e) {
	  System.out.println("about to close...");
	  //System.out.println("window closing...");
	  
	  /* to be investigated... see destroyThreads() for more info
	  confFile.getEditor().destroyThreads();
	  confFile.getViewer().destroyThreads();
	  */
	  System.exit(0); 
  }

  // window events that don't need an implementation (but required by the WindowListener interface)
  public void windowOpened(WindowEvent e) {}   public void windowClosed(WindowEvent e) {}
  public void windowIconified(WindowEvent e) {}  public void windowDeiconified(WindowEvent e) {}
  public void windowActivated(WindowEvent e) {}  public void windowDeactivated(WindowEvent e) {}
	 
  /** 
	* Displays the results of the Grail+ command execution in the right-hand side output panel
	* Obtains its data from the temp output file created by the shell command execution. 
	* 
	* @param outputFilePath the location where the temp output file can be found
	*/ 
  private void showResults(String outputFilePath) {
    try {
      BufferedReader reader = new BufferedReader(new FileReader(outputFilePath) );
      JTextArea taOutput    = getRightPanel().getOutputPanel().getOutputTextArea();

      taOutput.read(reader, null);
    }
    catch (IOException ioe) {
      System.out.println("io exception....");
    }
  }

    
	/** 
	* Prints the size of the given component to the screen, used for debugging.
	* 
	* @param c the component whose sizes will be printed 
	* @param cmpName the name of the component
	*/ 
  public void printSizeValues(JComponent c, String cmpName) {
     if ( c != null ) {
        System.out.println();
        System.out.println("----- " + cmpName + " ------");
        
       System.out.println("c.h = " + c.getSize().getHeight());
      	System.out.println("c.w = " + c.getSize().getWidth());

      	System.out.println("c.pref_h = " + c.getPreferredSize().getHeight());
      	System.out.println("c.pref_w = " + c.getPreferredSize().getWidth());

      	System.out.println("c.max_h = " + c.getMaximumSize().getHeight());
      	System.out.println("c.max_w = " + c.getMaximumSize().getWidth());
     }
  }


 /** 
  * Main function that runs the JGrail application. 
  * 
  * @param args the array of arguments, provided by Java.
  */  
  public static void main(String[] args) throws Exception {
    ConfFile confFile = new ConfFile();
	boolean parsed = false; 

	System.out.println("JGrail started...YAY");

	// run the setup program, to create the setup dir if need be
	try {
		Setup s = new Setup(); 
		
		//create the setup directory under /home/$USER, if need be
		s.create();
	}
	
	// show the "Found Bug" error dialog here...
	catch (Exception e) {
		Global.showBugErrorDialog(e); 
		
		System.exit(0);
	}

	// keep trying to parse the conffile until successful or user quits the program (via parseErrorDialog)	
	while ( !parsed ) {
		try {
			// parse the conf file 
			confFile.parseConfFile();
			
			// if we get here, then we've successfully parsed
			parsed = true; 
		}
		// catch any exceptions thrown by the conf file
		catch (ConfFileException cfe) {
			ConfFileParseErrorDialog dlg = new ConfFileParseErrorDialog(cfe.getMessage() );
			
			// show the dialog
			int result = dlg.showDlg();
			
			// user has decided to manually investigate the conf file, so close the program
			if ( result == ConfFileParseErrorDialog.MANUAL_BTN ) {
				System.exit(0);
			}				
		}
		
		// show the "Found Bug" error dialog here...
		catch (Exception e) {
			Global.showBugErrorDialog(e); 
			System.exit(1);
		}
	}
		
	// with the conf file successfully parsed, now show JGrail...    
    JGrailGui frame = new JGrailGui(confFile);
    try {
       frame.show();
    }

	// show the "Found Bug" error dialog here...
	catch (Exception e) {
		Global.showBugErrorDialog(e); 
		System.exit(1);
	}
  }



   // GETTERS and SETTERS // 


  // AGAINST THE STANDARD -- overriding/overloading issues //
  protected void setGrailMenuBar(GrailMenuBar menuBar) { this.menuBar = menuBar; }
  protected GrailMenuBar getGrailMenuBar()             { return this.menuBar; } 

  protected void setStatusBar(StatusBar statusBar) { this.statusBar = statusBar; }
  public StatusBar getStatusBar()               { return this.statusBar; } // PUBLIC -- used by grailmenubar //

  protected void setComputeErrorMsg(String msg) { this.computeErrorMsg = msg; }
  protected String getComputeErrorMsg()         { return this.computeErrorMsg; }

  protected void setMainPanel(JPanel mainPanel) { this.mainPanel = mainPanel; }
  protected JPanel getMainPanel()               { return this.mainPanel; }

  protected void setLeftPanel(LeftPanel leftPanel)  { this.leftPanel = leftPanel; }
  public LeftPanel getLeftPanel()                { return this.leftPanel; }  // PUBLIC //

  protected void setRightPanel(RightPanel rightPanel) { this.rightPanel = rightPanel; }
  public RightPanel getRightPanel()                    { return this.rightPanel; }  // PUBLIC //

  protected void setConfFile(ConfFile confFile) { this.confFile = confFile; }
  protected ConfFile getConfFile()              { return this.confFile; }
  
  // right-click context menu
  protected void setPopupMenu(JPopupMenu popupMenu) { this.popupMenu = popupMenu; }
  protected JPopupMenu getPopupMenu() { return this.popupMenu; }
            
}



